/*
 * This code was generated by OpenJava System.
 */
package examples.lazybind;


import openjava.ptree.*;
import java.util.Hashtable;


/**
 * The LazyBinder class is a metaclass which enables the
 * function of 'enum' on OpenJava.
 *
 * @see openjava.Class
 */
public class LazyBinder extends openjava.Class
{

  /** new modifier */
  private static final String LAZY = "lazy";

  /**
     * Overrides to define a new keyword
     */
  public String[] getUserModifiers()
  {
    String[] ret = { LAZY };
    return new String[]{ LAZY };
  }

  /**
     * Overrides to detect lazy-binding field declaration.
     */
  protected MemberDeclarationList translateMemberDeclarationList( MemberDeclarationList members )
  throws PtreeException
  {
    for( int i = 0, len = members.getLength(); i < len; ++i ){
      MemberDeclaration mem = members.elementAt( i );
      if(!(mem instanceof FieldDeclaration)){
        continue;
      }
      FieldDeclaration field = (FieldDeclaration) mem;
      if(!field.getModifiers().contains( LAZY )){
        continue;
      }
      MethodDeclaration init = registerLazyField( field );
      members.addElement( init );
      field.setInitializer( new SpecialName( SpecialName.NULL ) );
      field.setModifiers( privateOnly );
    }
    return super.translateMemberDeclarationList( members );
  }

  /**
     * Overrides to detect an access to lazy-binding field.
     */
  protected Expression translateFieldAccess( FieldAccess fieldaccess )
  throws PtreeException
  {
    Identifier fname = fieldaccess.getName();
    Expression expr = fieldaccess.getExpression();
    MethodDeclaration md = getLazyInitializer( fname );
    if(md != null && expr instanceof SpecialName){
      SpecialName ref = (SpecialName) expr;
      if(expr == null || ref.equals( "this" )){
        System.err.println( "#detects lazy access to " + fname );
        return makeExpression( "this." + md.getName() + "()" );
      }
    }
    return super.translateFieldAccess( fieldaccess );
  }

  private Hashtable lazyfields = new Hashtable();

  /**
     * Register a lazy field.
     */
  protected MethodDeclaration registerLazyField( FieldDeclaration field )
  {
    MethodDeclaration init = makeGettingMethod( field );
    String name = field.getVariable().toString();
    lazyfields.put( name, init );
    return init;
  }

  /**
     * Obtains the access method for the lazy field.
     */
  protected MethodDeclaration getLazyInitializer( Identifier fieldname )
  {
    String name = fieldname.toString();
    return (MethodDeclaration) lazyfields.get( name );
  }

  /**
     * Generates a new method declaration which tests this field and
     * assign value to the field if the field is null.
     * <p>
     *
     * For the field :
     * <pre>
     * lazy Object fld = initF();
     * </pre>
     * this generates a method :
     * <pre>
     * private Object lazilyGet_fld() {
     *     if (fld == null)  fld = initF();
     *     return fld;
     * }
     * </pre>
     */
  private static MethodDeclaration makeGettingMethod( FieldDeclaration fdecl )
  {
    TypeSpecifier type = fdecl.getTypeSpecifier();
    Identifier name = fdecl.getVariable();
    VariableInitializer init = fdecl.getInitializer();
    MethodDeclaration mdecl = MethodDeclaration.make( "private " + type + " lazilyGet_" + name + "() {" + "if (" + name + "== null)" + name + "=" + init + ";" + "return " + name + ";" + "}" );
    return mdecl;
  }

  private static final ModifierList privateOnly = new ModifierList( new Modifier( Modifier.PRIVATE ) );

}


