/*
 * Class.java 1.0
 *
 *  May 14, 1997 by mich
 *  Sep 28, 1997 by bv
 *  Oct 11, 1997 by mich
 *  Jan  1, 1997 by mich  fixes return types of translate methods.
 *
 *  @version 1.0  last update:  Oct 11, 1997
 *  @author Michiaki Tatsubori
 */
package openjava;


import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import openjava.ptree.*;
import openjava.tools.WrappedClass;
import openjava.main.Scanner;


/**
 * The <code>Class</code> class will be the base class of metaclassess.
 * and includes translation methods.
 */
public class Class extends WrappedClass
{
  private TypeDeclaration  definition;
  private Vector           addcu = new Vector();
  
  /**
   * Allocates a new Class instance (metaobject).
   *
   */
  public Class()
  {
    this( null );
  }

  /**
   * Allocates a new Class instance (metaobject).
   *
   * @param  def  the type declaration.
   */
  public Class( TypeDeclaration def )
  {
    this( def, null );
  }
  
  /**
   * Allocates a new Class instance (metaobject).
   *
   * @param  def  the type declaration.
   * @param  metaargs  the meta arguments. This will be ignored.
   */
  public Class( TypeDeclaration def, Ptree metaargs )
  {
    this.definition = def;
    if(metaargs != null){
      warningMessage( "meta arguments are ignored." );
    }
    initializeMetaInfo();
  }

  private QualifiedName pack = null;
  private ImportStatementList imps = null;

  /**
   * Only internal use by system.
   *
   * @param  pack  the package name
   */
  public void setPackage( QualifiedName pack ) {
      this.pack = pack;
  }

  /**
   * Obtain the package of the compilation unit including this class.
   *
   * @return  the qualified name of the package.
   */
  public QualifiedName getPackage() {
      return this.pack;
  }

  /**
   * Only internal use by system.
   *
   * @param  imps  the import statements
   */
  public void setImported( ImportStatementList imps ) {
      this.imps = imps;
  }

  /**
   * Obtain the import statements of the compilation unit including this
   * class.
   * 
   * @return  the import statements
   */
  public ImportStatementList getImported() {
      return this.imps;
  }

  /*********/
  private Object metainfo[] = new Object[]
  { null, new Hashtable(), new Hashtable() };
  private static final String METAFIELD = "openjava_metainfomation";

  private void initializeMetaInfo()
  {
    String clazzname = this.getClass().getName();
    metainfo[0] = new Object[]{ "0.2.1", clazzname };
  }
  private void finalizeMetaInfo()
  {
    ArrayInitializer whole = new ArrayInitializer();
    {/* openjava infomations */
      ArrayInitializer o_ainit = new ArrayInitializer();
      Object o_infos[] = (Object []) metainfo[0];
      {/* version */
	String version = (String) o_infos[0];
	o_ainit.addElement( Literal.makeLiteral( version ) );
      }
      {/* metaclass name */
	String clazzname = (String) o_infos[1];
	o_ainit.addElement( Literal.makeLiteral( clazzname ) );
      }
      whole.addElement( o_ainit );
    }
    {/* ptree infomations */
      ArrayInitializer p_ainit = new ArrayInitializer();
      Hashtable infos = (Hashtable) metainfo[1];
      Enumeration keys = infos.keys();
      while(keys.hasMoreElements()){
	ArrayInitializer elem = new ArrayInitializer();
	String key = (String) keys.nextElement();
	elem.addElement( Literal.makeLiteral( key ) );
	Ptree p = (Ptree) infos.get( key );
	elem.addElement( Literal.makeLiteral( p.toFlattenString() ) );
	p_ainit.addElement( elem );
      }
      whole.addElement( p_ainit );
    }
    {/* string infomations */
      ArrayInitializer s_ainit = new ArrayInitializer();
      Hashtable infos = (Hashtable) metainfo[1];
      Enumeration keys = infos.keys();
      while(keys.hasMoreElements()){
	ArrayInitializer elem = new ArrayInitializer();
	String key = (String) keys.nextElement();
	elem.addElement( Literal.makeLiteral( key ) );
	String str = (String) infos.get( key );
	elem.addElement( Literal.makeLiteral( str ) );
	s_ainit.addElement( elem );
      }
      whole.addElement( s_ainit );
    }
    /* constructs meta member */
    try{
      MemberDeclaration mdecl = makeMemberDeclaration
	( "public static String " + METAFIELD + " =" + whole + ";" );
      this.getTypeDeclaration().getBody().addElement( mdecl );
    }catch( PtreeException ex ){
      mopErrorMessage( ex.toString(), "illegal metainfomation." );
    }
  }
  protected final Ptree putMetaInfo( String key, Ptree ptree )
  {
    Hashtable p_info = (Hashtable) metainfo[1];
    return (Ptree) p_info.put( key, ptree );
  }
  protected final String putMetaInfo( String key, String str )
  {
    Hashtable s_info = (Hashtable) metainfo[2];
    return (String) s_info.put( key, str );
  }
  private static final String getMetaInfoName( java.lang.Class clazz )
  {
    try{
      java.lang.reflect.Field metafield = clazz.getField( METAFIELD );
      Object minfos[] = (Object[]) metafield.get( null );
      Object oj_info[] = (Object[]) minfos[0];
      return (String) oj_info[0];
    }catch( Exception ex ){
      return null;
    }
  }
  private static final String getMetaInfoVersion(
      java.lang.Class clazz )
  {
    try{
      java.lang.reflect.Field metafield = clazz.getField( METAFIELD );
      Object minfos[] = (Object[]) metafield.get( null );
      Object oj_info[] = (Object[]) minfos[0];
      return (String) oj_info[1];
    }catch( Exception ex ){
      return null;
    }
  }
  protected static final String getMetaInfoPtree(
      java.lang.Class clazz,
      String key )
  {
    if(key == null)  return null;
    try{
      java.lang.reflect.Field metafield = clazz.getField( METAFIELD );
      Object minfos[] = (Object[]) metafield.get( null );
      Object p_info[] = (Object[]) minfos[1];
      for( int i = 0; i < p_info.length; i++ ){
	Object elem[] = (Object[]) p_info[i];
	String k = (String) elem[0];
	if(key.equals( k )){
	  return (String) elem[1];
	}
      }
      return (String) null;
    }catch( Exception ex ){
      return (String) null;
    }
  }
  protected static final String getMetaInfoString(
      java.lang.Class clazz,
      String key )
  {
    if(key == null)  return null;
    try{
      java.lang.reflect.Field metafield = clazz.getField( METAFIELD );
      Object minfos[] = (Object[]) metafield.get( null );
      Object s_info[] = (Object[]) minfos[2];
      for( int i = 0; i < s_info.length; i++ ){
	Object elem[] = (Object[]) s_info[i];
	String k = (String) elem[0];
	if(key.equals( k )){
	  return (String) elem[1];
	}
      }
      return (String) null;
    }catch( Exception ex ){
      return (String) null;
    }
  }

  /**
   * Gets the additional compilation units.
   * This will be final method.
   * 
   * @return  the CompilationUnits newly added.
   */
  public CompilationUnit[] additionalCompilationUnits()
  {
    int len = addcu.size();
    CompilationUnit ret[] = new CompilationUnit[len];

    for( int i = 0; i < len; i++ ){
      ret[i] = (CompilationUnit) addcu.elementAt( i );
    }

    return ret;
  }

  /**
   * Adds the additional compilation unit to be generated.
   *
   * @param  cu  compilation unit.
   */
  public void addCompilationUnit( CompilationUnit cu )
  {
    addcu.addElement( cu );
  }

  /**
   * Tests if two ptree point one object.
   * This is utility method.
   *
   * @param  p  the ptree object to test.
   * @param  q  the ptree object to test.
   * @return  true if two ptree objects are same object.
   */
  public static final boolean eq( Ptree p, Ptree q )
  {
    return PtreeObject.eq( p, q );
  }

  /**
   * Tests if two ptree objects has same contents.
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @param  q  ptree-node object to test
   * @return  true  if two ptree objects has same contents
   */
  public static final boolean equal( Ptree p, Ptree q )
  {
    return PtreeObject.equal( p, q );
  }

  /**
   * Tests if the ptree object is an instance of iteration statement
   * classes.
   * <br>
   * Iteration statement classes are :
   * <br><blockquote>
   *   DoWhileStatement
   *   ForStatement
   *   WhileStatement
   * </blockquote><br>
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @return  true  if the specified ptree object is an instance of
   *                iteration statement classes.
   */
  public static final boolean isIterationStatement( Ptree p )
  {
    if(p instanceof DoWhileStatement)  return true;
    if(p instanceof ForStatement)  return true;
    if(p instanceof WhileStatement)  return true;
    return false;
  }

  /**
   * Tests if the ptree object is an instance of selection statement
   * classes.
   * <br>
   * Selection statement classes are :
   * <br><blockquote>
   *   IfStatement
   *   SwitchStatement
   * </blockquote><br>
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @return  true  if the specified ptree object is an instance of
   *                selection statement classes.
   */
  public static final boolean isSelectionStatement( Ptree p )
  {
    if(p instanceof IfStatement)  return true;
    if(p instanceof SwitchStatement)  return true;
    return false;
  }

  /**
   * Tests if the ptree object is an instance of jump statement
   * classes.
   * <br>
   * Jump statement classes are :
   * <br><blockquote>
   *   BreakStatement
   *   ContinueStatement
   *   ReturnStatement
   *   ThrowStatement
   * </blockquote><br>
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @return  true  if the specified ptree object is an instance of
   *                jump statement classes.
   */
  public static final boolean isJumpStatement( Ptree p )
  {
    if(p instanceof BreakStatement)  return true;
    if(p instanceof ContinueStatement)  return true;
    if(p instanceof ReturnStatement)  return true;
    if(p instanceof ThrowStatement)  return true;
    return false;
  }

  /**
   * Tests if the ptree object is an instance of guarding statement
   * classes.
   * <br>
   * Guarding statement classes are :
   * <br><blockquote>
   *   SynchronizedStatement
   *   TryStatement
   * </blockquote><br>
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @return  true  if the specified ptree object is an instance of
   *                guarding statement classes.
   */
  public static final boolean isGuardingStatement( Ptree p )
  {
    if(p instanceof SynchronizedStatement)  return true;
    if(p instanceof TryStatement)  return true;
    if(p instanceof Block)  return true;
    return false;
  }

  /**
   * Tests if the ptree object is an instance of allocation expression
   * classes.
   * <br>
   * Allocation expression classes are :
   * <br><blockquote>
   *   ConstructExpression
   *   ArrayAllocationExpression
   * </blockquote><br>
   * This is an utility method.
   *
   * @param  p  ptree-node object to test
   * @return  true  if the specified ptree object is an instance of
   *                allocation expression classes.
   */
  public static final boolean isAllocationExpression( Ptree p )
  {
    if(p instanceof ConstructExpression)  return true;
    if(p instanceof ArrayAllocationExpression)  return true;
    return false;
  }

  /**
   * Gets the type declaration.
   *
   * @return  the type declaration.
   */
  public final TypeDeclaration getTypeDeclaration()
  {
    return this.definition;
  }

  /**
   * Sets the type declaration.
   *
   * @param  td  the type declaration.
   */
  public final void setTypeDeclaration( TypeDeclaration td )
  {
    this.definition = td;
    initializeMetaInfo();
  }
  
  /**
   * Shows error messages and terminates OpenJava.
   *
   */
  public static void errorMessage( String str, Ptree keyword )
  {
    System.err.println( "error :" + str );
    System.err.println( keyword.toString() );
    System.exit( 0 );
  }
  
  /**
   * Shows an error message and terminates OpenJava.
   *
   */
  public static void errorMessage( String str )
  {
    System.err.println( str );
    System.exit( 0 );
  }

  /**
   * Shows error messages of MOP and terminates OpenJava.
   *
   */
  public static void mopErrorMessage( String str1, String str2 )
  {
    System.err.println( "mop error :" + str1 );
    System.err.println( str2 );
    System.exit( 0 );
  }

  /**
   * Shows an warning message.
   *
   */
  public static void warningMessage( String str )
  {
    System.err.println( str );
  }
  
  /**
   * Returns the user defined keywords.
   *
   */
  public String[] getUserKeywords()
  {
    return new String[0];
  }

  /**
   * Returns the user defined keywords.
   *
   */
  public String[] getUserModifiers()
  {
    return new String[0];
  }

  /**
   * Main method to be called by OpenJava system.
   *
   * @exception  PtreeException  when exception occurs
   */
  public TypeDeclaration translate()
    throws PtreeException
  {
    TypeDeclaration typedecl = getTypeDeclaration();
    initialize();
    typedecl = translateTypeDeclaration( typedecl );
    typedecl.setMetaclass( null );
    finish();
    setTypeDeclaration( typedecl );
    return typedecl;
  }

  /**
   * At the beginning, initialize() is once called on each metaclass.
   * The subclasses of Class may define their own Initialize().
   *
   */
  protected void initialize()
  {}
  
  /**
   * finish() is called on each metaobject at the end.
   *
   */
  protected void finish()
  {}
  
  /**
   * Translates type declaration
   *
   * @return translated node
   * @param typedecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected TypeDeclaration
  translateTypeDeclaration( TypeDeclaration typedecl )
    throws PtreeException
  {
    if(typedecl instanceof InterfaceDeclaration){
       return
         translateInterfaceDeclaration( (InterfaceDeclaration) typedecl );
    }
    if(typedecl instanceof ClassDeclaration){
       return
	 translateClassDeclaration( (ClassDeclaration) typedecl );
    }

    throw new PtreeException( "translateTypeDeclaration() : " +
			      "illegal TypeDeclaration object." );
  }
  
  /**
   * Translates interface declaration
   *
   * @return translated node
   * @param ifacedecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected TypeDeclaration
  translateInterfaceDeclaration( InterfaceDeclaration ifacedecl )
    throws PtreeException
  {
    ModifierList modiflist = ifacedecl.getModifiers();
    ifacedecl.setModifiers( translateModifierList( modiflist ) );
    
    Identifier name = ifacedecl.getName();
    ifacedecl.setName( translateIdentifier( name ) );
    
    ClassTypeList extendlist = ifacedecl.getSupers(); 
    ifacedecl.setSupers( translateClassTypeList( extendlist ) );
    
    MemberDeclarationList body = ifacedecl.getBody();
    ifacedecl.setBody( translateMemberDeclarationList( body ) );

    return ifacedecl;
  }
  
  /**
   * Translates class declaration
   *
   * @return translated node
   * @param classdecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected TypeDeclaration
  translateClassDeclaration( ClassDeclaration classdecl )
    throws PtreeException
  {
    ModifierList modifs = classdecl.getModifiers();
    classdecl.setModifiers( translateModifierList( modifs ) );
    
    Identifier name = classdecl.getName();
    classdecl.setName( translateIdentifier( name ) );
    
    ClassType zuper = classdecl.getSuper();
    zuper = (zuper == null) ? null : translateClassType( zuper );
    classdecl.setSuper( zuper );
    
    ClassTypeList ifacelist = classdecl.getInterfaces();
    classdecl.setInterfaces( translateClassTypeList( ifacelist ) );
    
    MemberDeclarationList body = classdecl.getBody();
    classdecl.setBody( translateMemberDeclarationList( body ) );

    return classdecl;
  }
  
  /**
   * Translates field declaration list
   *
   * @return translated node
   * @param fieldlist translating list
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclarationList
  translateMemberDeclarationList( MemberDeclarationList memlist )
    throws PtreeException
  {
    boolean same = true;
    MemberDeclarationList ret = new MemberDeclarationList();

    for( int i = 0; i < memlist.getLength(); i++ ){
      MemberDeclaration old_mdecl = memlist.elementAt( i );
      MemberDeclaration mdecl = translateMemberDeclaration( old_mdecl );
      if(! old_mdecl.eq( mdecl ))  same = false;
      ret.addElement( mdecl );
    }

    if(same){
      return memlist;
    }
    return ret;
  }
  
  /**
   * Translates member declaration list node
   *
   * @return  translated node
   * @param  mem  translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateMemberDeclaration( MemberDeclaration mem )
    throws PtreeException
  {
    if(mem instanceof FieldDeclaration){
      return
	  translateFieldDeclaration( (FieldDeclaration) mem );
    }
    if(mem instanceof MethodDeclaration){
      return
	  translateMethodDeclaration( (MethodDeclaration) mem );
    }
    if(mem instanceof ConstructorDeclaration){
      return
	  translateConstructorDeclaration( (ConstructorDeclaration) mem );
    }
    if(mem instanceof StaticInitializer){
      return
	  translateStaticInitializer( (StaticInitializer) mem );
    }
    if(mem instanceof InstanceInitializer){
      return
	  translateInstanceInitializer( (InstanceInitializer) mem );
    }
    if(mem instanceof TypeDeclaration){
      return
	  translateTypeDeclaration( (TypeDeclaration) mem );
    }
    throw new PtreeException( "translateMemberDeclaration() : " +
			      "illegal MemberDeclaration object." );
  }

  /**
   * Translates field variable declaration node
   *
   * @return translated node
   * @param fieldvardecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateFieldDeclaration( FieldDeclaration fielddecl )
    throws PtreeException
  {
    ModifierList old_modiflist = fielddecl.getModifiers();
    ModifierList modiflist = translateModifierList( old_modiflist );
    fielddecl.setModifiers( modiflist );
    
    TypeSpecifier old_type = fielddecl.getTypeSpecifier();
    TypeSpecifier type = translateTypeSpecifier( old_type );
    fielddecl.setTypeSpecifier( type );
    
    VariableDeclarator old_decl = fielddecl.getVariableDeclarator();
    VariableDeclarator decl = translateVariableDeclarator( old_decl );
    fielddecl.setVariableDeclarator( decl );

    return fielddecl;
  }
  
  /**
   * Translates method declaration node
   *
   * @return translated node
   * @param methoddecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateMethodDeclaration( MethodDeclaration methoddecl )
    throws PtreeException
  {
    ModifierList old_modiflist = methoddecl.getModifiers();
    ModifierList modiflist = translateModifierList( old_modiflist);
    methoddecl.setModifiers( modiflist );
    
    TypeSpecifier old_typespec = methoddecl.getTypeSpecifier();
    TypeSpecifier typespec = translateTypeSpecifier( old_typespec );
    methoddecl.setTypeSpecifier( typespec );
    
    Identifier old_name = methoddecl.getName();
    Identifier name = translateIdentifier( old_name );
    methoddecl.setName( name );

    ParameterList old_params = methoddecl.getParameters();
    ParameterList params = translateParameterList( old_params );
    methoddecl.setParameters( params );
    
    ClassTypeList old_throwlist = methoddecl.getThrows();
    ClassTypeList throwlist = translateClassTypeList( old_throwlist );
    methoddecl.setThrows( throwlist );
    
    StatementList old_body = methoddecl.getBody();
    StatementList body = null;
    if(old_body != null){
      body = translateStatementList( old_body );
    }
    methoddecl.setBody( body );

    return methoddecl;
  }
  
  /**
   * Translates constructor declaration node
   *
   * @return translated node
   * @param constdecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateConstructorDeclaration( ConstructorDeclaration constdecl )
    throws PtreeException
  {
    ModifierList old_modiflist = constdecl.getModifiers();
    ModifierList modiflist = translateModifierList( old_modiflist );
    constdecl.setModifiers( modiflist );

    Identifier old_name = constdecl.getName();
    Identifier name = translateIdentifier( old_name );
    constdecl.setName( name );

    ParameterList old_params = constdecl.getParameters();
    ParameterList params = translateParameterList( old_params );
    constdecl.setParameters( params );
    
    ClassTypeList old_throwlist = constdecl.getThrows();
    ClassTypeList throwlist = translateClassTypeList( old_throwlist );
    constdecl.setThrows( throwlist );
    
    StatementList old_body = constdecl.getBody();
    StatementList body = null;
    if(old_body != null){
      body = translateStatementList( old_body );
    }
    constdecl.setBody( body );

    return constdecl;
  }
  
  /**
   * Translates static initializer node
   *
   * @return translated node
   * @param staticinit translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateStaticInitializer( StaticInitializer staticinit )
    throws PtreeException
  {
    StatementList old_body = staticinit.getBody();
    StatementList body = translateStatementList( old_body );
    staticinit.setBody( body );

    return staticinit;
  }

  /**
   * Translates static initializer node
   *
   * @return translated node
   * @param staticinit translating node
   * @exception  PtreeException  when exception occurs
   */
  protected MemberDeclaration
  translateInstanceInitializer( InstanceInitializer insinit )
    throws PtreeException
  {
    StatementList old_body = insinit.getBody();
    StatementList body = translateStatementList( old_body );
    insinit.setBody( body );

    return insinit;
  }
  
  /**
   * Translates variable declarator node
   *
   * @return translated node
   * @param vardecl translating node
   * @exception  PtreeException  when exception occurs
   */
  protected VariableDeclarator
  translateVariableDeclarator( VariableDeclarator vardecl )
    throws PtreeException
  {
    Identifier old_declname = vardecl.getVariable();
    Identifier declname = translateIdentifier( old_declname );
    vardecl.setVariable( declname );

    VariableInitializer old_varinit = vardecl.getInitializer();
    VariableInitializer varinit = null;
    if(old_varinit != null){
      varinit = translateVariableInitializer( old_varinit );
    }
    vardecl.setInitializer( varinit );

    return vardecl;
  }
  
  /**
   * Translates the variable initializer node
   *
   * @return  translated node
   * @param  varinit  translating node
   * @exception  PtreeException  when exception occurs
   */
  protected VariableInitializer
  translateVariableInitializer( VariableInitializer varinit )
    throws PtreeException
  {
    if(varinit instanceof Expression){
      return
	  translateExpression( (Expression) varinit );
    }
    if(varinit instanceof ArrayInitializer){
      return
	  translateArrayInitializer( (ArrayInitializer) varinit );
    }

    throw new PtreeException( "translateVariableInitializer() : " +
			      "illegal VariableInitializer object." );
  }

  /**
   * Translates the array initializer node
   *
   * @return  translated node
   * @param  arrayinit  translating node
   * @exception  PtreeException  when exception occurs
   */
  protected ArrayInitializer
  translateArrayInitializer( ArrayInitializer arrayinit )
    throws PtreeException
  {
    boolean same = true;
    ArrayInitializer ret = new ArrayInitializer();

    for( int i = 0; i < arrayinit.getLength(); i++ ){
      VariableInitializer old_vinit = arrayinit.elementAt( i );
      VariableInitializer vinit = null;
      if(old_vinit != null){
	vinit = translateVariableInitializer( old_vinit );
      }
      if(! eq( old_vinit, vinit ))  same = false;
      ret.addElement( vinit );
    }

    if(same){
      return arrayinit;
    }
    return ret;
  }
  
  /**
   * Translates parameter list
   *
   * @return translated node
   * @param paramlist translating node
   * @exception  PtreeException  when exception occurs
   */
  protected ParameterList
  translateParameterList( ParameterList paramlist )
    throws PtreeException
  {
    boolean same = true;
    ParameterList ret = new ParameterList();

    for( int i = 0; i < paramlist.getLength(); i++ ){
      Parameter old_param = paramlist.elementAt( i );
      Parameter param = translateParameter( old_param );
      if(! old_param.eq( param ))  same = false;
      ret.addElement( param );
    }

    if(same){
      return paramlist;
    }
    return ret;
  }
  
  /**
   * Translates parameter
   *
   * @return translated node
   * @param param translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Parameter
  translateParameter( Parameter param )
    throws PtreeException
  {
    ModifierList old_modiflist = param.getModifiers();
    ModifierList modiflist = translateModifierList( old_modiflist );
    param.setModifiers( modiflist );
    
    TypeSpecifier old_typespec = param.getTypeSpecifier();
    TypeSpecifier typespec = translateTypeSpecifier( old_typespec );
    param.setTypeSpecifier( typespec );
    
    Identifier old_name = param.getVariable();
    Identifier name = translateIdentifier( old_name );
    param.setVariable( name );

    return param;
  }
  
  /**
   * Translates type specifier node
   *
   * @return translated node
   * @param typespec translating node
   * @exception  PtreeException  when exception occurs
   */
  protected TypeSpecifier
  translateTypeSpecifier( TypeSpecifier typespec )
    throws PtreeException
  {
    TypeName old_typename = typespec.getTypeName();
    TypeName typename = translateTypeName( old_typename );
    typespec.setTypeName( typename );
    
    int dim = typespec.getDim();
    typespec.setDim( dim );

    return typespec;
  }
  
  /**
   * Translates class type list
   *
   * @return  translated node
   * @param  ctypelist  translating node
   * @exception  PtreeException  when exception occurs
   */
  protected ClassTypeList
  translateClassTypeList( ClassTypeList ctypelist )
    throws PtreeException
  {
    boolean same = true;
    ClassTypeList ret = new ClassTypeList();

    for( int i = 0; i < ctypelist.getLength(); i++ ){
      ClassType old_ctype = ctypelist.elementAt( i );
      ClassType ctype = translateClassType( old_ctype );
      if(! old_ctype.eq( ctype ))  same = false;
      ret.addElement( ctype );
    }

    if(same){
      return ctypelist;
    }
    return ret;
  }
  
  /**
   * Translates type name node
   *
   * @return translated node
   * @param typename translating node
   * @exception  PtreeException  when exception occurs
   */
  protected TypeName
  translateTypeName( TypeName typename )
    throws PtreeException
  {
    if(typename instanceof PrimitiveType){
      return
	  translatePrimitiveType( (PrimitiveType) typename );
    }
    if(typename instanceof ClassType){
      return
	  translateClassType( (ClassType) typename );
    }

    throw new PtreeException( "translateTypeName() : " +
			      "illegal TypeName object." );
  }

  /**
   * Translates primitive type node
   *
   * @return translated node
   * @param typename translating node
   * @exception  PtreeException  when exception occurs
   */
  protected PrimitiveType
  translatePrimitiveType( PrimitiveType ptype )
    throws PtreeException
  {
    return ptype;
  }  

  /**
   * Translates class type node
   *
   * @return translated node
   * @param typename translating node
   * @exception  PtreeException  when exception occurs
   */
  protected ClassType
  translateClassType( ClassType ctypename )
    throws PtreeException
  {
    QualifiedName old_pack = ctypename.getPackage();
    QualifiedName pack = translateQualifiedName( old_pack );
    ctypename.setPackage( pack );

    Identifier old_name = ctypename.getName();
    Identifier name = translateIdentifier( old_name );
    ctypename.setName( name );
    
    return ctypename;
  }
  
  /**
   * Translates qualified name
   *
   * @return translated list
   * @param qname translating list
   * @exception  PtreeException  when exception occurs
   */
  protected QualifiedName
  translateQualifiedName( QualifiedName qname )
    throws PtreeException
  {
    boolean same = true;
    QualifiedName ret = new QualifiedName();

    for( int i = 0; i < qname.getLength(); i++ ){
      Identifier old_ident = qname.elementAt( i );
      Identifier ident = translateIdentifier( old_ident );
      if(! old_ident.eq( ident ))  same = false;
      ret.addElement( ident );
    }

    if(same){
      return qname;
    }
    return ret;
  }
  
  /**
   * Translates modifier list
   *
   * @return translated list
   * @param modiflist translating list
   * @exception  PtreeException  when exception occurs
   */
  protected ModifierList
  translateModifierList( ModifierList modiflist )
    throws PtreeException
  {
    boolean same = true;
    ModifierList ret = new ModifierList();

    for( int i = 0; i < modiflist.getLength(); i++ ){
      Modifier old_modif = modiflist.elementAt( i );
      Modifier modif = translateModifier( old_modif );
      if(! old_modif.eq( modif ))  same = false;
      ret.addElement( modif );
    }

    if(same){
      return modiflist;
    }
    return ret;
  }
  
  /**
   * Translates modfier node
   *
   * @return translated node
   * @param modfier translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Modifier translateModifier( Modifier modifier )
    throws PtreeException
  {
    return modifier;
  }

  /**
   * Translates block node
   *
   * @return translated node
   * @param block translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateBlock( Block block )
    throws PtreeException
  {
    StatementList old_stmts = block.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    block.setStatements( stmts );
    
    return block;
  }
  
  /**
   * Translates local variable declaration statement node.
   *
   * @return  translated node
   * @param  vardeclstate  translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateVariableDeclaration( VariableDeclaration vardecl )
    throws PtreeException
  {
    ModifierList old_modiflist = vardecl.getModifiers();
    ModifierList modiflist = translateModifierList( old_modiflist );
    vardecl.setModifiers( modiflist );

    TypeSpecifier old_typespec = vardecl.getTypeSpecifier();
    TypeSpecifier typespec = translateTypeSpecifier( old_typespec );
    vardecl.setTypeSpecifier( typespec );
    
    VariableDeclarator old_vd = vardecl.getVariableDeclarator();
    VariableDeclarator vd = translateVariableDeclarator( old_vd );
    vardecl.setVariableDeclarator( vd );

    return vardecl;
  }
  
  /**
   * Translates statement list
   *
   * @return translated node
   * @param paramlist translating node
   * @exception  PtreeException  when exception occurs
   */
  protected StatementList
  translateStatementList( StatementList stmtlist )
    throws PtreeException
  {
    boolean same = true;
    StatementList ret = new StatementList();

    for( int i = 0; i < stmtlist.getLength(); i++ ){
      Statement old_stmt = stmtlist.elementAt( i );
      Statement stmt = translateStatement( old_stmt );
      if(! old_stmt.eq( stmt )){
	same = false;
      }
      ret.addElement( stmt );
    }

    if(same){
      return stmtlist;
    }
    return ret;
  }

  /**
   * Translates statement node
   *
   * @return translated node
   * @param statement translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateStatement( Statement stmt )
    throws PtreeException
  {
    if(stmt instanceof VariableDeclaration){
      return
	translateVariableDeclaration( (VariableDeclaration) stmt );
    }
    if(stmt instanceof EmptyStatement){
      return
	translateEmptyStatement( (EmptyStatement) stmt );
    }
    if(stmt instanceof LabeledStatement){
      return
	translateLabeldStatement( (LabeledStatement) stmt );
    }
    if(stmt instanceof ExpressionStatement){
      return
	translateExpressionStatement( (ExpressionStatement) stmt );
    }
    if(isSelectionStatement( stmt )){
      return
	translateSelectionStatement( stmt );
    }
    if(isIterationStatement( stmt )){
      return
	translateIterationStatement( stmt );
    }
    if(isJumpStatement( stmt )){
      return
	translateJumpStatement( stmt );
    }
    if(isGuardingStatement( stmt )){
      return
	translateGuardingStatement( stmt );
    }
    throw new PtreeException( "translateStatement() : " +
			      "illegal Statement object." );
  }
  
  /**
   * Translates empty statement
   *
   * @return translated node
   * @param emptystate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateEmptyStatement( EmptyStatement emptystate )
    throws PtreeException
  {
    return emptystate;
  }
    
  /**
   * Translates labeled statement node
   *
   * @return  translated node.
   * @param  labelstate  translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateLabeldStatement( LabeledStatement labelstate )
    throws PtreeException
  {
    Identifier old_label = labelstate.getLabel();
    Identifier label = translateIdentifier( old_label );
    labelstate.setLabel( label );

    Statement old_stmt = labelstate.getStatement();
    Statement stmt = translateStatement( old_stmt );
    labelstate.setStatement( stmt );

    return labelstate;
  }
  
  /**
   * Translates expression statement node.
   *
   * @return  translated node.
   * @param  expstate  translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateExpressionStatement( ExpressionStatement expstate )
    throws PtreeException
  {
    Expression old_expr = expstate.getExpression();
    Expression expr = translateExpression( old_expr );
    expstate.setExpression( expr );

    return expstate;
  }
  
  /**
   * Translates selection statement node
   *
   * @return translated node
   * @param selectstate translating ndoe
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateSelectionStatement( Statement  selectstate )
    throws PtreeException
  {
    if(selectstate instanceof IfStatement){
      return
	  translateIfStatement( (IfStatement) selectstate);
    }
    if(selectstate instanceof SwitchStatement){
      return
	  translateSwitchStatement( (SwitchStatement) selectstate);
    }

    throw new PtreeException( "translateSelectionStatement() : " +
			      "illegal selection statement object." );
  }
  
  /**
   * Translates if statement node
   *
   * @return translated node
   * @param ifstate translating ndoe
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateIfStatement( IfStatement ifstate )
    throws PtreeException
  {
    Expression old_expr = ifstate.getExpression();
    Expression expr = translateExpression( old_expr );
    ifstate.setExpression( expr );
    
    StatementList old_stmts = ifstate.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    ifstate.setStatements( stmts );
      
    StatementList old_elsestmts = ifstate.getElseStatements();
    StatementList elsestmts = translateStatementList( old_elsestmts );
    ifstate.setElseStatements( elsestmts );

    return ifstate;
  }
  
  /**
   * Translates switch statement node
   *
   * @return translated node
   * @param switchstate translating ndoe
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateSwitchStatement( SwitchStatement switchstate )
    throws PtreeException
  {
    Expression old_expr = switchstate.getExpression();
    Expression expr = translateExpression( old_expr );
    switchstate.setExpression( expr );
    
    CaseGroupList old_cglist = switchstate.getCaseGroupList();
    CaseGroupList cglist = translateCaseGroupList( old_cglist );
    switchstate.setCaseGroupList( cglist );

    return switchstate;
  }

  /**
   * Translates switch statement node
   *
   * @return translated node
   * @param switchstate translating ndoe
   * @exception  PtreeException  when exception occurs
   */
  protected CaseGroupList
  translateCaseGroupList( CaseGroupList cglist )
    throws PtreeException
  {
    boolean same = true;
    CaseGroupList ret = new CaseGroupList();
    
    for( int i = 0; i < cglist.getLength(); i++ ){
      CaseGroup old_cg = cglist.elementAt( i );
      CaseGroup cg = translateCaseGroup( old_cg );
      if(! old_cg.eq( cg ))  same = false;
      ret.addElement( cg );
    }
    
    if(same){
      return cglist;
    }
    return ret;
  }

  /**
   * Translates statements after case node
   *
   * @return translated node
   * @param switchstate translating ndoe
   * @exception  PtreeException  when exception occurs
   */
  protected CaseGroup
  translateCaseGroup( CaseGroup cg )
    throws PtreeException
  {
    return cg;
  }
  
  /**
   * Translates iteration node
   *
   * @return translated node
   * @param iterstate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateIterationStatement( Statement iterstate )
    throws PtreeException
  {
    if(iterstate instanceof WhileStatement){
      return translateWhileStatement( (WhileStatement) iterstate );
    }
    if(iterstate instanceof DoWhileStatement){
      return translateDoWhileStatement( (DoWhileStatement) iterstate );
    }
    if(iterstate instanceof ForStatement){
      return translateForStatement( (ForStatement) iterstate );
    }

    throw new PtreeException( "translateIterationStatement() : " +
			      "illegal IterationStatement object." );
  }
  
  /**
   * Translates while statement node
   *
   * @return translated node
   * @param whilestate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateWhileStatement( WhileStatement whilestate )
    throws PtreeException
  {
    Expression old_expr = whilestate.getExpression();
    Expression expr = translateExpression( old_expr );
    whilestate.setExpression( expr );
    
    StatementList old_stmts = whilestate.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    whilestate.setStatements( stmts );

    return whilestate;
  }
  
  /**
   * Translates the do-while statement node.
   *
   * @return translated node
   * @param dostate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateDoWhileStatement( DoWhileStatement dostate )
    throws PtreeException
  {
    StatementList old_stmts = dostate.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    dostate.setStatements( stmts );
    
    Expression old_expr = dostate.getExpression();
    Expression expr = translateExpression( old_expr );
    dostate.setExpression( expr );
    
    return dostate;
  }
  
  /**
   * Translates the for statement node.
   *
   * @return translated node
   * @param forstate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateForStatement( ForStatement forstate )
    throws PtreeException
  {
    List init = forstate.getInit();
    
    Expression old_expr = forstate.getCondition();
    Expression expr = null;
    if(old_expr != null){
      expr = translateExpression( old_expr );
    }
    forstate.setCondition( expr );
    
    ExpressionList old_incr = forstate.getIncrement();
    ExpressionList incr = translateExpressionList( old_incr );
    forstate.setIncrement( incr );

    StatementList old_stmts = forstate.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    forstate.setStatements( stmts );

    return forstate;
  }
  
  /**
   * Translates jump statement node
   *
   * @return translated node
   * @param jumpstate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateJumpStatement( Statement jumpstate )
    throws PtreeException
  {
    if(jumpstate instanceof BreakStatement){
      return translateBreakStatement( (BreakStatement) jumpstate );
    }
    if(jumpstate instanceof ContinueStatement){
      return translateContinueStatement( (ContinueStatement) jumpstate );
    }
    if(jumpstate instanceof ReturnStatement){
      return translateReturnStatement( (ReturnStatement) jumpstate );
    }
    if(jumpstate instanceof ThrowStatement){
      return translateThrowStatement( (ThrowStatement) jumpstate );
    }

    throw new PtreeException( "translateJumpStatement : " +
			      "illegal JumpStatement object." );
  }
  
  /**
   * Translates break statement node.
   *
   * @return  the translated node.
   * @param  breakstate  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateBreakStatement( BreakStatement breakstate )
    throws PtreeException
  {
    Identifier old_label = breakstate.getLabel();
    Identifier label = null;
    if(old_label != null){
      label = translateIdentifier( old_label );
    }
    breakstate.setLabel( label );

    return breakstate;
  }
  
  /**
   * Translates the continue statement node.
   *
   * @return  the translated node.
   * @param  continuestate  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateContinueStatement( ContinueStatement continuestate )
    throws PtreeException
  {
    Identifier old_label = continuestate.getLabel();
    Identifier label = null;
    if(old_label != null){
      label = translateIdentifier( old_label );
    }
    continuestate.setLabel( label );

    return continuestate;
  }
  
  /**
   * Translates return statement node
   *
   * @return translated node
   * @param returnstate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateReturnStatement( ReturnStatement returnstate )
    throws PtreeException
  {
    Expression old_expr = returnstate.getExpression();
    Expression expr = null;
    if(old_expr != null){
      expr = translateExpression( old_expr );
    }
    returnstate.setExpression( expr );

    return returnstate;
  }
  
  /**
   * Translates throw statement node
   *
   * @return translated node
   * @param throwstate translating node
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateThrowStatement( ThrowStatement throwstate )
    throws PtreeException
  {
    Expression old_expr = throwstate.getExpression();
    Expression expr = null;
    if(old_expr != null){
      expr = translateExpression( old_expr );
    }
    throwstate.setExpression( expr );

    return throwstate;
  }
  
  /**
   * Translates the guarding statement node.
   *
   * @return  the translated node.
   * @param  guardstate  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateGuardingStatement( Statement guardstate )
    throws PtreeException
  {
    if(guardstate instanceof SynchronizedStatement){
      return
	translateSynchronizedStatement( (SynchronizedStatement) guardstate );
    }
    if(guardstate instanceof TryStatement){
      return translateTryStatement( (TryStatement) guardstate );
    }
    if(guardstate instanceof Block){
      return translateBlock( (Block) guardstate );
    }

    throw new PtreeException( "translateGuardingStatement() : " +
			      "illegal GuardingStatement object." );
  }
  
  /**
   * Translates synchronized statement node.
   *
   * @return  the translated node.
   * @param  synctate  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateSynchronizedStatement( SynchronizedStatement syncstate )
    throws PtreeException
  {
    Expression old_expr = syncstate.getExpression();
    Expression expr = translateExpression( old_expr );
    syncstate.setExpression( expr );
    
    StatementList old_stmts = syncstate.getStatements();
    StatementList stmts = translateStatementList( old_stmts );
    syncstate.setStatements( stmts );

    return syncstate;
  }
  
  /**
   * Translates the try statement node.
   *
   * @return  the translated node.
   * @param  trystate  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Statement
  translateTryStatement( TryStatement trystate )
    throws PtreeException
  {
    StatementList old_stmts = trystate.getBody();
    StatementList stmts = translateStatementList( old_stmts );
    trystate.setBody( stmts );
    
    CatchList old_catches = trystate.getCatchList();
    CatchList catches = translateCatchList( old_catches );
    trystate.setCatchList( catches );
    
    StatementList old_finallie = trystate.getFinallyBody();
    StatementList finallie = translateStatementList( old_finallie );
    trystate.setFinallyBody( finallie );

    return trystate;
  }
  
  /**
   * Translates catch list
   *
   * @return  translated node
   * @param  catchlist translating node
   * @exception  PtreeException  when exception occurs
   */
  protected CatchList
  translateCatchList( CatchList catchlist )
    throws PtreeException
  {
    boolean same = true;
    CatchList ret = new CatchList();

    for( int i = 0; i < catchlist.getLength(); i++ ){
      CatchBlock old_cb = catchlist.elementAt( i );
      CatchBlock cb = translateCatchBlock( old_cb );
      if(! old_cb.eq( cb ))  same = false;
      ret.addElement( cb );
    }

    if(same){
      return catchlist;
    }
    return ret;
  }
  
  /**
   * Translates the catch block node.
   *
   * @return  the translated node.
   * @param  katch  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected CatchBlock
  translateCatchBlock( CatchBlock katch )
    throws PtreeException
  {
    Parameter old_param = katch.getParameter();
    Parameter param = translateParameter( old_param );
    katch.setParameter( param );
    
    StatementList old_stmts = katch.getBody();
    StatementList stmts = translateStatementList( old_stmts );
    katch.setBody( stmts );

    return katch;
  }
  
  /**
   * Translates the expression list.
   *
   * @return  the translated node.
   * @param  exprlist  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected ExpressionList
  translateExpressionList( ExpressionList exprlist )
    throws PtreeException
  {
    boolean same = true;
    ExpressionList ret = new ExpressionList();

    for( int i = 0; i < exprlist.getLength(); i++ ){
      Expression old_expr = exprlist.elementAt( i );
      Expression expr = translateExpression( old_expr );
      if(! old_expr.eq( expr ))  same = false;
      ret.addElement( expr );
    }

    if(same){
      return exprlist;
    }
    return ret;
  }
  
  /**
   * Translates the expression node.
   *
   * @return  the translated node.
   * @param  expr  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateExpression( Expression expr )
    throws PtreeException
  {
    if(expr instanceof UnaryExpression){
      return
	  translateUnaryExpression( (UnaryExpression) expr );
    }
    if(expr instanceof BinaryExpression){
      return
	  translateBinaryExpression( (BinaryExpression) expr );
    }
    if(expr instanceof InstanceofExpression){
      return
	  translateInstanceofExpression( (InstanceofExpression) expr );
    }
    if(expr instanceof ConditionalExpression){
      return
	  translateConditionalExpression( (ConditionalExpression) expr );
    }
    if(expr instanceof AssignmentExpression){
      return
	  translateAssignmentExpression( (AssignmentExpression) expr );
    }
    if(expr instanceof CastExpression){
      return
	  translateCastExpression( (CastExpression) expr );
    }
    if(isAllocationExpression( expr )){
      return
	  translateAllocationExpression( expr );
    }
    if(expr instanceof QualifiedName){
      return
	  translateQualifiedName( (QualifiedName) expr );
    }
    if(expr instanceof SpecialName){
      return
	  translateSpecialName( (SpecialName) expr );
    }
    if(expr instanceof Literal){
      return
	  translateLiteral( (Literal) expr );
    }
    if(expr instanceof ArrayAccess){
      return
	  translateArrayAccess( (ArrayAccess) expr );
    }
    if(expr instanceof FieldAccess){
      return
	  translateFieldAccess( (FieldAccess) expr );
    }
    if(expr instanceof MethodCall){
      return
	  translateMethodCall( (MethodCall) expr );
    }
    throw new PtreeException( "translateExpression() : " +
			      "illegal Expression object." );
  }

  /**
   * Translates the literal.
   *
   * @return  the translated node.
   * @param  lit  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateLiteral( Literal lit )
    throws PtreeException
  {
    return lit;
  }

  /**
   * Translates the unary expression.
   *
   * @return  the translated node.
   * @param  unary  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateUnaryExpression( UnaryExpression unary )
    throws PtreeException
  {
    int opr = unary.getOperator();
    
    Expression old_expr = unary.getExpression();
    Expression expr = translateExpression( old_expr );
    unary.setExpression( expr );

    return unary;
  }
  
  /**
   * Translates the binary expression.
   *
   * @return  the translated node.
   * @param  binary  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateBinaryExpression( BinaryExpression binary )
    throws PtreeException
  {
    Expression old_left = binary.getLeft();
    Expression left = translateExpression( old_left );
    binary.setLeft( left );
    
    int opr = binary.getOperator();
    
    Expression old_right = binary.getRight();
    Expression right = translateExpression( old_right );
    binary.setRight( right );

    return binary;
  }

  /**
   * Translates the instanceof expression.
   *
   * @return  the translated node.
   * @param  insof  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateInstanceofExpression( InstanceofExpression insof )
    throws PtreeException
  {
    Expression old_left = insof.getLeft();
    Expression left = translateExpression( old_left );
    insof.setLeft( left );
    
    TypeSpecifier old_right = insof.getTypeSpecifier();
    TypeSpecifier right = translateTypeSpecifier( old_right );
    insof.setTypeSpecifier( right );

    return insof;
  }
  
  /**
   * Translates the conditional expression.
   *
   * @return  the translated node.
   * @param  condexpr  the translationg node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateConditionalExpression( ConditionalExpression condexpr )
    throws PtreeException
  {
    Expression old_condition = condexpr.getCondition();
    Expression condition = translateExpression( old_condition );
    condexpr.setCondition( condition );
    
    Expression old_truecase = condexpr.getTrueCase();
    Expression truecase = translateExpression( old_truecase );
    condexpr.setTrueCase( truecase );
    
    Expression old_falsecase = condexpr.getFalseCase();
    Expression falsecase = translateExpression( old_falsecase );
    condexpr.setTrueCase( falsecase );

    return condexpr;
  }
  
  /**
   * Translates assignment expression node
   *
   * @return translated node
   * @param assignment translationg node
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateAssignmentExpression( AssignmentExpression assignment )
    throws PtreeException
  {
    Expression old_left = assignment.getLeft();
    Expression left = translateExpression( old_left );
    assignment.setLeft( left );
    
    int opr = assignment.getOperator();
    
    Expression old_right = assignment.getRight();
    Expression right = translateExpression( old_right );
    assignment.setRight( right );

    return assignment;
  }
  
  /**
   * Translates the cast expression node.
   *
   * @return  the translated node.
   * @param  cast  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateCastExpression( CastExpression castexpr )
    throws PtreeException
  {
    TypeSpecifier old_ts = castexpr.getTypeSpecifier();
    TypeSpecifier ts = translateTypeSpecifier( old_ts );
    castexpr.setTypeSpecifier( ts );
    
    Expression old_expr = castexpr.getExpression();
    Expression expr = translateExpression( old_expr );
    castexpr.setExpression( expr );

    return castexpr;
  }
  
  /**
   * Translates the allocation expression node.
   *
   * @return  the translated node.
   * @param  alloc  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateAllocationExpression( Expression alloc )
    throws PtreeException
  {
    if(alloc instanceof ConstructExpression){
      return
	  translateConstructExpression( (ConstructExpression) alloc );
    }
    if(alloc instanceof ArrayAllocationExpression){
      return
	  translateArrayAllocationExpression( (ArrayAllocationExpression) alloc );
    }
    throw new PtreeException( "translateAllocationExpression() : " +
			      "illegal AllocationExpression object." );
  }
  
  /**
   * Translates the construct expression node.
   *
   * @return  the translated node.
   * @param  construct  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateConstructExpression( ConstructExpression construct )
    throws PtreeException
  {
    ClassType old_typename = construct.getClassType();
    ClassType typename = translateClassType( old_typename );
    construct.setClassType( typename );
    
    ExpressionList old_exprs = construct.getArguments();
    ExpressionList exprs = translateExpressionList( old_exprs );
    construct.setArguments( exprs );

    MemberDeclarationList old_body = construct.getClassBody();
    MemberDeclarationList body = null;
    if(old_body != null){
      body = translateMemberDeclarationList( old_body );
    }

    return construct;
  }
  
  /**
   * Translates the array allocation expression node.
   *
   * @return the translated node.
   * @param  arrayalloc  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateArrayAllocationExpression( ArrayAllocationExpression arrayalloc )
    throws PtreeException
  {
    TypeName old_typename = arrayalloc.getTypeName();
    TypeName typename = translateTypeName( old_typename );
    arrayalloc.setTypeName( typename );
    
    DimExprList old_dims = arrayalloc.getDimExprList();
    DimExprList dims = translateDimExprList( old_dims );
    arrayalloc.setDimExprList( dims );

    ArrayInitializer old_ainit = arrayalloc.getInitializer();
    ArrayInitializer ainit = null;
    if(old_ainit != null){
      ainit = translateArrayInitializer( old_ainit );
    }
    arrayalloc.setInitializer( ainit );

    return arrayalloc;
  }

  /**
   * Translates the array dimension expr list.
   *
   * @return translated node
   * @param construct translating node
   * @exception  PtreeException  when exception occurs
   */
  protected DimExprList
  translateDimExprList( DimExprList delist )
    throws PtreeException
  {
    boolean same = true;
    DimExprList ret = new DimExprList();

    for( int i = 0; i < delist.getLength(); i++ ){
      Expression old_expr = delist.elementAt( i );
      Expression expr = null;
      if(old_expr != null){
	expr = translateExpression( old_expr );
      }
      if(! eq( old_expr, expr ))  same = false;
      ret.addElement( expr );
    }

    if(same){
      return delist;
    }
    return ret;
  }
  
  /**
   * Translates the array access node.
   *
   * @return the translated node.
   * @param  arrayaccess  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateArrayAccess( ArrayAccess arrayaccess )
    throws PtreeException
  {
    Expression old_expr = arrayaccess.getExpression();
    Expression expr = translateExpression( old_expr );
    arrayaccess.setExpression( expr );
    
    DimExprList old_dims = arrayaccess.getDimExprList();
    DimExprList dims = translateDimExprList( old_dims );
    arrayaccess.setDimExprList( dims );

    return arrayaccess;
  }
  
  /**
   * Translates the field access node.
   *
   * @return  the translated node.
   * @param  fieldaccess  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateFieldAccess( FieldAccess fieldaccess )
    throws PtreeException
  {
    Expression old_expr = fieldaccess.getExpression();
    Expression expr = null;
    if(old_expr != null){
      expr = translateExpression( old_expr );
    }
    fieldaccess.setExpression( expr );
    
    Identifier old_name = fieldaccess.getName();
    Identifier name = translateIdentifier( old_name );
    fieldaccess.setName( name );

    return fieldaccess;
  }
  
  /**
   * Translates the mehtod call node.
   *
   * @return  the translated node.
   * @param  methodcall  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateMethodCall( MethodCall methodcall )
    throws PtreeException
  {
    Expression old_expr = methodcall.getExpression();
    Expression expr = null;
    if(old_expr != null){
      expr = translateExpression( old_expr );
    }
    methodcall.setExpression( expr );

    Identifier old_name = methodcall.getName();
    Identifier name = translateIdentifier( old_name );
    methodcall.setName( name );
    
    ExpressionList old_args = methodcall.getArguments();
    ExpressionList args = null;
    args = translateExpressionList( old_args );
    methodcall.setArguments( args );

    return methodcall;
  }
  
  /**
   * Translates the special name node.
   *
   * @return  the translated node.
   * @param  special  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Expression
  translateSpecialName( SpecialName special )
    throws PtreeException
  {
    return special;
  }

  /**
   * Translates the identifier node.
   *
   * @return  the translated node.
   * @param  id  the translating node.
   * @exception  PtreeException  when exception occurs
   */
  protected Identifier
  translateIdentifier( Identifier id )
    throws PtreeException
  {
    return id;
  }
  
}
