/*
 * GUIMain.java 1.0
 *
 * The class to start OpenJava with GUI
 *
 * Mar  12, 1998 mich 
 *
 * @version  last update:  Mar 12, 1998
 * @author Michiaki Tatsubori
 */
package openjava.main;


import java.lang.*;
import java.util.*;
import openjava.main.Main;
import java.applet.Applet;
import java.awt.Frame;
import java.awt.TextField;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

import java.awt.Component;
import java.awt.Panel;
import java.awt.Label;
import java.awt.Checkbox;
import java.awt.List;
import java.awt.FileDialog;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;


/**
 * The Main class is a main class to start OpenJava
 *
 */
public class GUIMain
{
  public static void main( String[] args ){
    Frame frame = new OJFrame();
  }

}


class OJFrame extends java.awt.Frame
{
  private TextField commandOption = null;
  private GUISelection selection = null;

  OJFrame(){
    super( "OpenJava System GUI" );

    this.addWindowListener( new OJWindowAdapter() );

    Panel panel = new Applet();

    this.selection = new GUISelection( this );
    panel.add( selection );

    panel.add( new Label( " other options:" ) );
    panel.add( this.commandOption = new TextField( 30 ) );

    {
      Button runButton = new Button( "Run" );
      runButton.addActionListener( new OJButtonAdapter() );
      panel.add( runButton );
    }

    {
      Button quitButton = new Button( "Quit" );
      quitButton.addActionListener( new OJButtonAdapter() );
      panel.add( quitButton );
    }

    this.setSize( new Dimension( 500, 300 ) );
    this.add( panel );
    this.show();
    this.doLayout();
  }

  class OJButtonAdapter implements ActionListener
  {
    public void actionPerformed( ActionEvent e ) {
      String command = e.getActionCommand();
      if(command == "Run"){
	System.out.println( "Run" );
	String[] options = selection.getArguments();
	openjava.main.Main.main( options );
	return;
      }
      if(command == "Quit"){
	System.out.println( "Quit" );
	dispose();
	System.exit( 1 );
	return;
      }
    }
  }

  class OJWindowAdapter extends java.awt.event.WindowAdapter
  {
    public void windowClosing( WindowEvent e ) {
      dispose();
      System.exit( 1 );
    }
  }  

}


class GUISelection extends Panel
{
  private static final int CK_COMPILE = 0;
  private static final int CK_STDOUT  = 1;
  private static final int CK_VERBOSE = 2;
  private static final int CK_NOTRANS = 3;
  private static final int CK_NOFILE  = 4;

  private Checkbox[] check = {
    new Checkbox( "-c\tCompile" ),
    new Checkbox( "-s\tOutput into stdout" ),
    new Checkbox( "-v\tVerbose output" ),
    new Checkbox( "-t\tno Translation" ),
    new Checkbox( "-f\tno output File" )
  };
  private TextField  compilerCommand = new TextField( 20 );
  private List fileList = new List( 4, true );
  private Button addFileButton    = new Button( "Add File" );
  private Button removeFileButton = new Button( "Remove File" );

  private FileDialog fileDialog = null;

  /**
   * Constructs this panel.
   */
  public GUISelection( Frame parentFrame ) {
    super();
    fileDialog = new FileDialog( parentFrame, "Choose a file to add" );
    addFileButton.addActionListener( new ButtonAdapter() );
    removeFileButton.addActionListener( new ButtonAdapter() );
    initLayout();
  }

  private void initLayout() {
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    this.setFont( new Font("Helvetica", Font.PLAIN, 12) );
    this.setLayout( gridbag );
    c.anchor = GridBagConstraints.WEST;

    c.fill = GridBagConstraints.BOTH;
    c.weightx = 0.0;
    setComponent( gridbag, c,         fileList, 0, 0, 3, 5 );
    setComponent( gridbag, c, new Label( "Files to be translated." ), 0, 0 );
    setComponent( gridbag, c, removeFileButton, 1, 5 );
    setComponent( gridbag, c,    addFileButton, 2, 5 );

    c.fill = GridBagConstraints.BOTH;
    c.weightx = 0.0;
    setComponent( gridbag, c, new Label( "   " ), 3, 0 );

    c.fill = GridBagConstraints.VERTICAL;
    c.weightx = 1.0;
    setComponent( gridbag, c, check[CK_COMPILE], 4, 0 );
    setComponent( gridbag, c, compilerCommand,   4, 1 );
    setComponent( gridbag, c, check[CK_STDOUT ], 4, 2 );
    setComponent( gridbag, c, check[CK_VERBOSE], 4, 3 );
    setComponent( gridbag, c, check[CK_NOTRANS], 4, 4 );
    setComponent( gridbag, c, check[CK_NOFILE ], 4, 5 );

  }

  private void setComponent( GridBagLayout       gridbag,
			     GridBagConstraints  c,
			     Component           comp,
			     int x, int y, int width, int height )
  {
    c.gridx = x;
    c.gridy = y;
    c.gridwidth = width;
    c.gridheight = height;
    gridbag.setConstraints( comp, c );
    add( comp );
  }
  private void setComponent( GridBagLayout       gblayout,
			     GridBagConstraints  c,
			     Component           comp,
			     int x, int y )
  {
    setComponent( gblayout, c, comp, x, y, 1, 1 );
  }

  public String[] getArguments() {
    Vector tmp_argv = new Vector();
    if(check[CK_COMPILE].getState()){
      tmp_argv.addElement( "-c" + compilerCommand.getText() );
    }
    if(check[CK_STDOUT ].getState())  tmp_argv.addElement( "-s" );
    if(check[CK_VERBOSE].getState())  tmp_argv.addElement( "-v" );
    if(check[CK_NOTRANS].getState())  tmp_argv.addElement( "-t" );
    if(check[CK_NOFILE ].getState())  tmp_argv.addElement( "-f" );

    String[] files = fileList.getItems();
    for( int i = 0; i < files.length; i++ ){
      tmp_argv.addElement( files[i] );
    }

    //here other options

    String[] argv = new String[tmp_argv.size()];
    Enumeration argv_i = tmp_argv.elements();
    for( int i = 0; argv_i.hasMoreElements(); i++ ){
      argv[i] = (String) argv_i.nextElement();
    }

    return argv;
  }

  class ButtonAdapter implements ActionListener
  {
    public void actionPerformed( ActionEvent e ) {
      String command = e.getActionCommand();
      if(command == "Remove File"){
	System.out.println( "Remove File" );
	int index;
	while((index = fileList.getSelectedIndex()) >= 0){
	  fileList.remove( index );
	}
	return;
      }
      if(command == "Add File"){
	System.out.println( "Add File" );
	fileDialog.show();
	fileList.addItem( fileDialog.getDirectory() + fileDialog.getFile() );
	return;
      }
    }
  }

}

