/*
 * Main.java 1.0
 *
 * The class to start OpenJava
 *
 * Jun  1, 1997 mich
 * Aug 29, 1997 mich
 * Oct 14, 1997 mich
 * Jan  9, 1998 mich  Added the function invoking compiler 
 * Jul 29, 1998 mich, Fixed the part adding new compilation unit
 *
 * @version  last update:  Oct 14, 1997
 * @author Michiaki Tatsubori
 */
package openjava.main;


import java.lang.Exception;
import java.lang.String;
import java.lang.System;
import java.lang.Runtime;
import java.io.File;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.Writer;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.Enumeration;
import openjava.*;
import openjava.ptree.*;


/**
 * The Main class is a main class to start OpenJava
 *
 */
public class Main
{

    public static void main( String argv[] ) {
	Arguments args;
	try {
	    args = new Arguments( argv );
	    try {
		new Openjava( args ).run();
	    } catch( Exception ex ) {
		System.err.println( "OpenJava Exception: " + ex.getMessage() );
		ex.printStackTrace();
		System.exit( -1 );
	    }
	} catch	( ArgumentsException e ) {
	    System.err.println( e.toString() );
	}
    }

    public static final void showHelp() {
	System.out.println( "OpenJava usage :" );
	System.out.println
	    (
	     "% java openjava.main.Main " +
	     "[-c[compiler]] [-l[N]] [-s] [-v] [-t] [-f] [-h]" +
	     "source." + Openjava.SUFFIX_OJ +
	     " [source2." + Openjava.SUFFIX_OJ + " .. ]" 
	     );
	System.out.println( " Option :" );
	System.out.println
	    ( "-c[compiler]  Specifys the compiler command for byte codes." );
	System.out.println
	    ( "-l[N]  Inserts the information about ptree nodes to debug." );
	System.out.println
	    ( "   N   Specifies debug level. Default is 0." );
	System.out.println
	    ( "-s  Outputs the result into standard output." );
	System.out.println
	    ( "-v  Show system status messages." );
	System.out.println
	    ( "-t  Doesn't translate source files." );
	System.out.println
	    ( "-f  Doesn't output into files" );
	System.out.println
	    ( "-h  Show this message." );
    }

}


class Arguments
{

    Vector files = new Vector();

    /* followings are for debug */
    boolean debugList = false;
    boolean debugStdout = false;
    boolean debugVerbose = false;
    boolean debugTrans = true;
    boolean debugFile = true;
    int debugLevel = 0;
    PrintWriter DEBUG_OUT = new PrintWriter( System.out );
    boolean optionCompile = false;
    String optionCompiler = null;

    Arguments( String argv[] )
	throws ArgumentsException
    {
	super();
	init( argv );
	checkNames();
	if (this.files.size() == 0) {
	    Main.showHelp();
	    throw new ArgumentsException( "no file" );
	}

    }

    private void init( String argv[] )
	throws ArgumentsException
    {
	for (int i = 0; i < argv.length; ++i) {
	    if (argv[i].charAt(0) != '-') {
		files.addElement( new File( argv[i] ) );
		continue;
	    }
	    
	    switch (argv[i].charAt(1)) {
	    case 'c' :
		optionCompile = true;
		if(argv[i].length() < 3)  break;
		optionCompiler = argv[i].substring( 2 );
	    case 'l' :
		debugList = true;
		if (argv[i].length() < 3)  break;
		switch (argv[i].charAt(2)) {
		case '0' :  this.debugLevel = 0;  break;
		case '1' :  this.debugLevel = 1;  break;
		case '2' :  this.debugLevel = 2;  break;
		case '3' :  this.debugLevel = 3;  break;
		case '4' :  this.debugLevel = 3;  break;
		case '5' :  this.debugLevel = 3;  break;
		default  :  this.debugLevel = 0;
		}
		break;
	    case 's' :
		debugStdout = true;  break;
	    case 'v' :
		debugVerbose = true;  break;
	    case 't' :
		debugTrans = false;  break;
	    case 'f' :
		debugFile = false;  break;
	    case 'h' :
		Main.showHelp();
		throw new ArgumentsException( "" );
	    }
	}
    }

    private void checkNames()
	throws ArgumentsException
    {
	for (int i = 0; i < files.size(); ++i) {
	    File file = (File) files.elementAt( i );
	    if (! file.exists()) {
		throw new ArgumentsException( "file not found : " + file );
	    }
	    if (! file.isFile() || ! file.canRead()) {
		throw new ArgumentsException( "invalid file : " + file );
	    }
	    if (! file.getName().endsWith( Openjava.SUFFIX_OJ )) {
		System.err.println( "filename must end with " +
				   "\"" + Openjava.SUFFIX_OJ + "\" ." );
		throw new ArgumentsException( "invalid file : " + file );
	    }
	}
    }

}


class ArgumentsException extends Exception
{
    ArgumentsException( String str ) {
	super( str );
    }
}


class Openjava
{
    public static final String SUFFIX_OJ = ".oj";
    public static final String SUFFIX_JAVA = ".java";
    
    private Vector addcu = new Vector();
    private File currentFile = null;
    private Vector generatedFiles = new Vector();


    private Arguments args;

    public Openjava( Arguments args ) {
	this.args = args;
    }

    /**
     * Runs OpenJava.
     */
    public void run() throws Exception {
	Enumeration files = args.files.elements();
	while (files.hasMoreElements()) {
	    /* set input to file */
	    this.currentFile = (File) files.nextElement();
	    InputStream fin = new FileInputStream( currentFile );
	    InputStream in = new BufferedInputStream( fin );
	    ojtranslator( in );
	}

	if (args.optionCompile)  javac();

	/* finish */
	if (args.debugVerbose) {
	    System.err.println( "... finished." );
	}
    }

    /**
     * Generate parse tree from input and translates it.
     */
    void ojtranslator( InputStream in ) throws Exception {
	/* allocate a parser object */
	Parser parse_obj = new Parser( in );
	parse_obj.setDebug( args.debugLevel );
	System.err.println( "Reading from " + this.currentFile );
	    
	/* parsing */
	if (args.debugVerbose) {
	    System.err.println( "parsing ... " );
	    parse_obj.debug_parse();
	} else {
	    parse_obj.parse();
	}
	translateAndOutput( parse_obj.getParseTree() );
	
	/* additional compilation units */
	Enumeration cus = this.addcu.elements();
	while (cus.hasMoreElements()) {
	    CompilationUnit cu = (CompilationUnit) cus.nextElement();
	    String name = cu.getPublicClass().getName().toString();
	    this.currentFile
		= new File( this.currentFile.getParent(),
			   name + SUFFIX_OJ );
	    translateAndOutput( cu );
	}
    }

    private void translateAndOutput( CompilationUnit cu )
	throws Exception
    {
	/* translation */
	if (args.debugVerbose) {
	    System.err.println( "translating parse-tree ... " );
	}
	if (args.debugTrans) {
	    translateCompilationUnit( cu );
	}

	/* sets debug options */
	if (args.debugList) {
	    cu.setDebugFlag( true );
	    cu.setDebugLevel( args.debugLevel );
	} else {
	    cu.setDebugFlag( false );
	}

	/* output */
	if (args.debugVerbose) {
	    System.err.println( "generating source code ... " );
	}
	if (args.debugStdout) {
	    cu.pushPrintWriter( args.DEBUG_OUT );
	    cu.writeCode();
	    cu.popPrintWriter();
	}
	if (args.debugFile) {
	    File file = obtainOutputFile( cu );
	    this.generatedFiles.addElement( file );

	    Writer out = new BufferedWriter( new FileWriter( file ) );
	    PrintWriter writer = new PrintWriter( out );
	    
	    cu.pushPrintWriter( writer );
	    cu.writeCode();
	    cu.popPrintWriter();
	}
    }

    /**
     * Obtains the file name to output.
     */
    private File obtainOutputFile( CompilationUnit cu )
	throws PtreeException
    {
	String simplename = this.currentFile.getName();
	String dir = this.currentFile.getParent();
	
	int endindex = simplename.length() - SUFFIX_OJ.length();
	String simpleoutname
	    = simplename.substring( 0, endindex ) + SUFFIX_JAVA;

	String pubname = cu.getPublicClass().getName().toString();

	return new File( dir, pubname + SUFFIX_JAVA );
    }

    protected void translateCompilationUnit( CompilationUnit cu )
	throws PtreeException
    {
	TypeDeclarationList tdlst = cu.getTypeDeclarations();
	for (int i = 0; i < tdlst.getLength(); ++i) {
	    TypeDeclaration td
		= translateTypeDeclaration( cu.getPackage(),
					    tdlst.elementAt( i ) );
	    tdlst.setElementAt( td, i );
	}
    }

    protected TypeDeclaration
    translateTypeDeclaration( QualifiedName pack, TypeDeclaration td )
	throws PtreeException
    {
	System.err.println( "translating class " + td.getName() );
	
	openjava.Class clazz;
	
	/* continue translation until the metaclass is set to null */
	while (((clazz = td.getMetaclass()) != null)
	       && (! clazz.getClass().getName().equals( "openjava.Class" ))) {
	    clazz.setTypeDeclaration( td );
	    clazz.setPackage( pack );
	    System.err.println( "  with " + clazz.getClass().getName() );
	    td = clazz.translate();
	    
	    CompilationUnit addcu[] = clazz.additionalCompilationUnits();
	    for (int j = 0; j < addcu.length; ++j) {
		this.addcu.addElement( addcu[j] );
	    }
	}

	/* default class translation */
	if (args.debugList && args.debugLevel > 2) {
	    clazz = new openjava.Class( td );
	    clazz.setPackage( pack );
	    System.err.println( "translating with " +
			       clazz.getClass().getName() + ". (debug mode)" );
	    td = clazz.translate();
	}

	/* member classes */
	MemberDeclarationList members = td.getBody();
	for (int i = 0; i < members.getLength(); ++i) {
	    MemberDeclaration m = members.elementAt( i );
	    if (! (m instanceof TypeDeclaration))  continue;
	    TypeDeclaration itd
		= translateTypeDeclaration( pack, (TypeDeclaration) m );
	    members.setElementAt( itd, i );
	}
	
	return td;
    }

    /**
     * Compiles the generated files into byte codes
     */
    void javac() {
	if (args.debugVerbose) {
	    System.err.println( "compiling ..." );
	}
	if (args.optionCompiler == null) {
	    String[] filenames = new String[generatedFiles.size()];
	    for( int i = 0; i < filenames.length; i++ ){
		File file = (File) generatedFiles.elementAt(i);
		filenames[i] = file.getPath();
	    }
	    sunjavac( filenames );
	} else {
	    Enumeration srcfiles = this.generatedFiles.elements();
	    StringBuffer filenames = new StringBuffer();
	    while (srcfiles.hasMoreElements()) {
		File file = (File) srcfiles.nextElement();
		filenames.append( file.getPath() + " " );
	    }
	    try {
		exec( args.optionCompiler, filenames.toString() );
	    } catch( Exception ex ) {
		System.err.println( ex );
	    }
	}
    }

    static void sunjavac( String[] filenames ) {
	try { /* for 1.0, 1.1 */
	    java.lang.Class clazz
		= java.lang.Class.forName( "sun.tools.javac.Main" );
	    java.lang.reflect.Method method
		= clazz.getMethod( "main", new java.lang.Class[]{ String[] . class } );
	    method.invoke( null, new Object[]{ filenames } );
	    return;
	} catch ( ClassNotFoundException e ) {
	} catch ( NoSuchMethodException e0 ) {
	    System.err.println( e0 );
	} catch ( IllegalAccessException e1 ) {
	    System.err.println( e1 );
	} catch ( IllegalArgumentException e2 ) {
	    System.err.println( e2 );
	} catch ( java.lang.reflect.InvocationTargetException e3 ) {
	    System.err.println( e3 );
	    e3.printStackTrace();
	}
	
        try {
	    exec( "javac", strs2str( filenames ) );
        } catch (Exception e) {
            System.err.println( e );
        }
    }

    private static String strs2str( String[] strs ) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < strs.length; ++i) {
            buf.append( strs[i] ).append( " " );
        }
        return buf.toString();
    }

    static void exec( String command, String args ) throws Exception {
	Runtime runtime = Runtime.getRuntime();
	Process p = runtime.exec( command + " " + args );
	InputStream in = new BufferedInputStream( p.getErrorStream() );
	byte[] buf = new byte[1024];
	for (int len = in.read( buf ); len != -1; len = in.read( buf ) ) {
	    System.err.write( buf, 0, len );
	}
	p.waitFor();
    }

}

