/*
 * Scanner.java 1.0
 *
 * @version 1.0  May 12, 1997
 * @author Michiaki Tatsubori
 *
 * Scanner Class
 */
package openjava.main;


import java.lang.System;
import java.io.InputStream;
import java.io.IOException;
import java.lang.reflect.*;
import openjava.ptree.Ptree;
import openjava.ptree.Leaf;


public class Scanner
{
  protected Yylex lexer;

  /** last token */
  protected Yytoken oldToken;

  /** present token */
  protected Yytoken presentToken;

  /** 1-lookahead token */
  protected Yytoken nextToken;

  /**
   * table for user's keyword
   */
  protected String userKeywords[] = new String[0];
  protected String userModifiers[] = new String[0];


  /**
   * Constructs this scanner.
   *
   */
  public Scanner()
  throws IOException {
    this( System.in );
  }

  /**
   * Constructs this scanner.
   *
   */
  public Scanner( InputStream in )
    throws IOException
  {
    lexer = new Yylex( in );
    oldToken = null;
    // dummy. just set not to be null
    presentToken = new Yytoken( SymbolTable.EOF );
    nextToken = lexer.yylex();
  }

  /**
   * Constructs this scanner.
   *
   */
  public Scanner( InputStream in, String keywords[] )
    throws IOException
  {
    this( in );
    userKeywords = keywords;
  }    

  /**
   * recognizes the next token and returns the token
   *
   */
  public Yytoken getToken()
    throws IOException
  {
    oldToken = presentToken;
    presentToken = nextToken;
    nextToken = lexer.yylex();
    if(nextToken == null)  nextToken = new Yytoken( SymbolTable.EOF );

    if(debugLevel > 1){
      System.err.println( " o:" + oldToken.sym +
			  " p:" + presentToken.sym +
			  " n:" + nextToken.sym +
			  "  p.text:" + presentToken.text );
      if(debugLevel > 3){
	if(presentToken.comments != null){
	  java.util.Vector vec = presentToken.comments;
	  for( int i = 0; i < vec.size(); i++ ){
	    System.err.println( vec.elementAt( i ) );
	  }
	}
      }
    }

    switch( presentToken.sym ){
    case SymbolTable.DOT :
      //*detect DOT-"." DOTCLASS-".class"
      switch( nextToken.sym ){
      case SymbolTable.CLASS :
	//***DOTCLASS
	presentToken = new Yytoken( SymbolTable.DOTCLASS,
				    nextToken.text,
				    presentToken );
	nextToken = lexer.yylex();
	if(debugLevel > 1){
	  System.err.println( "-> o:" + oldToken.sym +
			      " p:" + presentToken.sym +
			      " n:" + nextToken.sym +
			      "  p.text:" + presentToken.text );
	}
	return presentToken;
      }
      break;
    case SymbolTable.LBRACK :
      //*detect LBRACK-"[" OP_EMPTYDIM-"[]"
      switch( nextToken.sym ){
      case SymbolTable.RBRACK :
	//***OP_EMPTYDIM
	presentToken = new Yytoken( SymbolTable.OP_EMPTYDIM,
				    nextToken.text,
				    presentToken );
	nextToken = lexer.yylex();
	if(debugLevel > 1){
	  System.err.println( "-> o:" + oldToken.sym +
			      " p:" + presentToken.sym +
			      " n:" + nextToken.sym +
			      "  p.text:" + presentToken.text );
	}
	return presentToken;
      }
      break;
    case SymbolTable.IDENTIFIER :
      //*detect IDENTIFIER DEFINEDNAME USERMODIFIER
      for( int i = 0; i < userKeywords.length; i++ ){
	if(! presentToken.text.equals( userKeywords[i] ))  continue;
	//***USERKEYWORD
	presentToken = new Yytoken( SymbolTable.USERKEYWORD,
				    presentToken );
	if(debugLevel > 1){
	  System.err.println( "-> o:" + oldToken.sym +
			      " p:" + presentToken.sym +
			      " n:" + nextToken.sym +
			      "  p.text:" + presentToken.text );
	}
	return presentToken;
      }
      for( int i = 0; i < userModifiers.length; i++ ){
	if(! presentToken.text.equals( userModifiers[i] ))  continue;
	//***USERMODIFIER
	presentToken = new Yytoken( SymbolTable.USERMODIFIER,
				    presentToken );
	if(debugLevel > 1){
	  System.err.println( "-> o:" + oldToken.sym +
			      " p:" + presentToken.sym +
			      " n:" + nextToken.sym +
			      "  p.text:" + presentToken.text );
	}
	return presentToken;
      }
      break;
    }

    return presentToken;
  }

  public void setUserKeywords( String uk[] ) {
    System.err.print( "UserKeywords :" );
    if(uk == null)  uk = new String[0];
    for( int i = 0; i < uk.length; i++ ){
      System.err.print( " " + uk[i] );
    }
    System.err.println();

    userKeywords = uk;
  }

  public void setUserModifiers( String uk[] ) {
    System.err.print( "UserModifiers :" );
    if(uk == null)  uk = new String[0];
    for( int i = 0; i < uk.length; i++ ){
      System.err.print( " " + uk[i] );
    }
    System.err.println();

    userModifiers = uk;
  }

  public static int getTokenID( String str ) {
    try{
      Class stClazz = Class.forName( "openjava.main.SymbolTable" );
      Field fld = stClazz.getField( str );
      return fld.getInt( null );
    }catch( Exception ex ){
      System.err.println( "error : " + ex.toString() );
      return -1;
    }
  }

  protected int debugLevel = 0;

  public void setDebug( int level )
  {
    this.debugLevel = level;
  }

}

