/*
 * Yytoken.java 1.0
 *
 * This subclass of symbol represents (at least) terminal symbols returned 
 * by the scanner and placed on the parse stack.  At present, this 
 * class does nothing more than its super class.
 *
 * May 10, 1997 mich
 * Jun 11, 1997 mich
 * Oct 13, 1997 mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0  Oct 13, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.main;


import java.io.OutputStream;
import java.lang.String;
import java.lang.System;
import java.util.Vector;


/**
 * The Yytoken class makes a media for javacup.
 * Presents for token-node of parse tree.
 *
 */
class Yytoken extends java_cup.runtime.token
{
  public int sym = -1;
  public String text = null;
  public int line = -1;
  public int charBegin = -1;
  public Vector comments = null;

  /**
   * Allocates a token.
   *
   */
  public Yytoken( int index )
  {
    this( index, "", -2, -2 );
  }

  /**
   * Re-constructs a token object from the token with the specified
   * index symbol identifier.
   *
   */
  public Yytoken( int index, Yytoken old )
  {
    this( index, old.text, old.line, old.charBegin, old.comments );
  }

  /**
   * Re-constructs a token object from the token with the specified
   * index symbol identifier and the joined text.
   *
   */
  public Yytoken( int index, String text, Yytoken old )
  {
    this( index, old.text + text, old.line, old.charBegin, old.comments );
  }

  /**
   * Allocates a token.
   *
   */
  public Yytoken( int index, String text, int line, int charBegin )
  {
    this( index, text, line, charBegin, null );
  }

  /**
   * Allocates a token.
   *
   */
  public Yytoken( int index, String text, int line, int charBegin,
		  Vector comments )
  {
    super( index );
    this.sym = index;
    this.text = new String( text );
    this.line = line;
    this.charBegin = charBegin;
    this.comments = comments;
  }

}
