/*
 * openjava.lex 1.0
 *
 * The following will be resolved by the <code>Scanner</code> class.
 * "[]" can be "[  ]"
 * ".class" can be ".  class "
 *
 * @author Michiaki Tatsubori
 * @version 1.0  May 10, 1997
 */
package openjava.main;


import java.lang.String;
import java.lang.System;
import java.util.Vector;
import java.util.Enumeration;


%%


%init{
  commentNest = 0;
%init}

%{
  protected int           commentNest = 0;
  protected Vector        javadocComments = null;
  protected StringBuffer  commentLine = null;

  protected Yytoken mkRet( int tokenID ) {
    Yytoken ret = new Yytoken( tokenID, yytext(),
			       yyline, yychar,
			       javadocComments );
    javadocComments = null;
    return ret;
  }
  
  private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character.",
    "Error: Unexpected error."
  };
  
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 
  public static final int E_UNEXPECTED = 4; 

  public void error( int code )
  {
    System.err.println( errorMsg[code] );
    System.err.println( "text=" + yytext() +
                        " line=" + yyline + " char=" + yychar );    
    throw( new Error( "Lexical Error: assertion failed." ) );
  }

%} 

%public
%class		Yylex
%function	yylex
%type		Yytoken
%notunix
%full
%unicode

%eof{
  if(yy_lexical_state != YYINITIAL){
    error( E_ENDCOMMENT );
  }
%eof}

%line
%char
%state COMMENT, DOCU, LINE

D=[0-9]
H=[a-fA-F0-9]
L=[a-zA-Z_$]
EXP=([Ee]["+""-"]?{D}+)
FS=(f|F|d|D)
IS=(l|L)
OD=[0-7]
ESC=(\\[btnfr\"'\\]|\\{OD}|\\{OD}{OD}|\\[0123]{OD}{OD})
UNI=(\\u*{H}{H}{H}{H})

BLANK=([\ \t\f\b]+)
LN=(\n|\r|\n\r)

%% 


<YYINITIAL> "abstract"		{ return mkRet( SymbolTable.ABSTRACT ); }
<YYINITIAL> "boolean"		{ return mkRet( SymbolTable.BOOLEAN ); }
<YYINITIAL> "break"		{ return mkRet( SymbolTable.BREAK ); }
<YYINITIAL> "byte"		{ return mkRet( SymbolTable.BYTE ); }
<YYINITIAL> "case"		{ return mkRet( SymbolTable.CASE ); }
<YYINITIAL> "catch"		{ return mkRet( SymbolTable.CATCH ); }
<YYINITIAL> "char"		{ return mkRet( SymbolTable.CHAR ); }
<YYINITIAL> "class"		{ return mkRet( SymbolTable.CLASS ); }
<YYINITIAL> "continue"		{ return mkRet( SymbolTable.CONTINUE ); }
<YYINITIAL> "default"		{ return mkRet( SymbolTable.DEFAULT ); }
<YYINITIAL> "do"		{ return mkRet( SymbolTable.DO ); }
<YYINITIAL> "double"		{ return mkRet( SymbolTable.DOUBLE ); }
<YYINITIAL> "else"		{ return mkRet( SymbolTable.ELSE ); }
<YYINITIAL> "extends"		{ return mkRet( SymbolTable.EXTENDS ); }
<YYINITIAL> "final"		{ return mkRet( SymbolTable.FINAL ); }
<YYINITIAL> "finally"		{ return mkRet( SymbolTable.FINALLY ); }
<YYINITIAL> "float"		{ return mkRet( SymbolTable.FLOAT ); }
<YYINITIAL> "for"		{ return mkRet( SymbolTable.FOR ); }
<YYINITIAL> "if"		{ return mkRet( SymbolTable.IF ); }
<YYINITIAL> "implements"	{ return mkRet( SymbolTable.IMPLEMENTS ); }
<YYINITIAL> "import"		{ return mkRet( SymbolTable.IMPORT ); }
<YYINITIAL> "instanceof"	{ return mkRet( SymbolTable.INSTANCEOF ); }
<YYINITIAL> "int"		{ return mkRet( SymbolTable.INT ); }
<YYINITIAL> "interface"		{ return mkRet( SymbolTable.INTERFACE ); }
<YYINITIAL> "long"		{ return mkRet( SymbolTable.LONG ); }
<YYINITIAL> "native"		{ return mkRet( SymbolTable.NATIVE ); }
<YYINITIAL> "new"		{ return mkRet( SymbolTable.NEW ); }
<YYINITIAL> "package"		{ return mkRet( SymbolTable.PACKAGE ); }
<YYINITIAL> "private"		{ return mkRet( SymbolTable.PRIVATE ); }
<YYINITIAL> "protected"		{ return mkRet( SymbolTable.PROTECTED ); }
<YYINITIAL> "public"		{ return mkRet( SymbolTable.PUBLIC ); }
<YYINITIAL> "return"		{ return mkRet( SymbolTable.RETURN ); }
<YYINITIAL> "short"		{ return mkRet( SymbolTable.SHORT ); }
<YYINITIAL> "static"		{ return mkRet( SymbolTable.STATIC ); }
<YYINITIAL> "super"		{ return mkRet( SymbolTable.SUPER ); }
<YYINITIAL> "switch"		{ return mkRet( SymbolTable.SWITCH ); }
<YYINITIAL> "synchronized"	{ return mkRet( SymbolTable.SYNCHRONIZED ); }
<YYINITIAL> "this"		{ return mkRet( SymbolTable.THIS ); }
<YYINITIAL> "throw"		{ return mkRet( SymbolTable.THROW ); }
<YYINITIAL> "throws"		{ return mkRet( SymbolTable.THROWS ); }
<YYINITIAL> "transient"		{ return mkRet( SymbolTable.TRANSIENT ); }
<YYINITIAL> "try"		{ return mkRet( SymbolTable.TRY ); }
<YYINITIAL> "void"		{ return mkRet( SymbolTable.VOID ); }
<YYINITIAL> "volatile"		{ return mkRet( SymbolTable.VOLATILE ); }
<YYINITIAL> "while"		{ return mkRet( SymbolTable.WHILE ); }

<YYINITIAL> "true"		{ return mkRet( SymbolTable.TRUE ); }
<YYINITIAL> "false"		{ return mkRet( SymbolTable.FALSE ); }
<YYINITIAL> "null"		{ return mkRet( SymbolTable.NULL ); }

<YYINITIAL> "const"		{ return mkRet( SymbolTable.CONST ); }
<YYINITIAL> "goto"		{ return mkRet( SymbolTable.GOTO ); }

<YYINITIAL> "metaobject"	{ return mkRet( SymbolTable.METAOBJECT ); }
<YYINITIAL> "//metaobject"	{ return mkRet( SymbolTable.METAOBJECT ); }


<YYINITIAL> {BLANK}		{ }
<YYINITIAL,COMMENT> {LN}	{ }

<YYINITIAL> "//"		{ yybegin(LINE); }
<LINE> .*{LN}			{ yybegin(YYINITIAL); }

<YYINITIAL> "/*"		{
	  yybegin(COMMENT);
	  commentNest++;
	}
<COMMENT> "/*"			{ commentNest++; }
<COMMENT> "*/"			{
	  commentNest--;
	  if(commentNest < 0)  error( E_UNEXPECTED );
	  if(commentNest == 0)  yybegin(YYINITIAL);
	}
<COMMENT> {LN}			{ }
<COMMENT> .			{ }

<YYINITIAL> "/**"		{
	  yybegin(DOCU);
	  commentNest++;
	  javadocComments = new Vector();
	  commentLine = new StringBuffer( yytext() );
	}
<DOCU> "/*"			{
	  commentNest++;
	  commentLine.append( yytext() );
	}
<DOCU> "*/"			{
	  commentNest--;
	  if(commentNest < 0)  error( E_UNEXPECTED );
	  commentLine.append( yytext() );
	  if(commentNest == 0){
	    javadocComments.addElement( commentLine.toString() );
	    commentLine = null;
	    yybegin(YYINITIAL);
          }
	}
<DOCU> {LN}			{
	  javadocComments.addElement( commentLine );
	  commentLine = new StringBuffer();
	}
<DOCU> .			{
	  commentLine.append( yytext() );
	}


<YYINITIAL> {L}({L}|{D})*	{ return mkRet( SymbolTable.IDENTIFIER ); }

<YYINITIAL> 0[xX]{H}+{IS}?	{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> 0{OD}+{IS}?		{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> {D}+{IS}?		{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> {D}+{EXP}{FS}?	{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> {D}+{EXP}?{FS}	{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> {D}*"."{D}+{EXP}?{FS}?	{ return mkRet( SymbolTable.NUMBER ); }
<YYINITIAL> {D}+"."{D}*{EXP}?{FS}?	{ return mkRet( SymbolTable.NUMBER ); }

<YYINITIAL> '({ESC}|{UNI}|[^\\'])'
				{ return mkRet( SymbolTable.CHARACTER ); }
<YYINITIAL> \"({ESC}|{UNI}|[^\\\"])*\"	
				{ return mkRet( SymbolTable.STRING ); }

<YYINITIAL> ">>="	{ return mkRet( SymbolTable.ASS_SHR ); }
<YYINITIAL> ">>>="	{ return mkRet( SymbolTable.ASS_SHRR ); }
<YYINITIAL> "<<="	{ return mkRet( SymbolTable.ASS_SHL ); }
<YYINITIAL> "+="	{ return mkRet( SymbolTable.ASS_ADD ); }
<YYINITIAL> "-="	{ return mkRet( SymbolTable.ASS_SUB ); }
<YYINITIAL> "*="	{ return mkRet( SymbolTable.ASS_MUL ); }
<YYINITIAL> "/="	{ return mkRet( SymbolTable.ASS_DIV ); }
<YYINITIAL> "%="	{ return mkRet( SymbolTable.ASS_MOD ); }
<YYINITIAL> "&="	{ return mkRet( SymbolTable.ASS_AND ); }
<YYINITIAL> "^="	{ return mkRet( SymbolTable.ASS_XOR ); }
<YYINITIAL> "|="	{ return mkRet( SymbolTable.ASS_OR ); }
<YYINITIAL> ">>"	{ return mkRet( SymbolTable.OP_SHR ); }
<YYINITIAL> ">>>"	{ return mkRet( SymbolTable.OP_SHRR ); }
<YYINITIAL> "<<"	{ return mkRet( SymbolTable.OP_SHL ); }
<YYINITIAL> "++"	{ return mkRet( SymbolTable.OP_INC ); }
<YYINITIAL> "--"	{ return mkRet( SymbolTable.OP_DEC ); }
<YYINITIAL> "&&"	{ return mkRet( SymbolTable.OP_LAND ); }
<YYINITIAL> "||"	{ return mkRet( SymbolTable.OP_LOR ); }
<YYINITIAL> "<="	{ return mkRet( SymbolTable.OP_LE ); }
<YYINITIAL> ">="	{ return mkRet( SymbolTable.OP_GE ); }
<YYINITIAL> "=="	{ return mkRet( SymbolTable.OP_EQ ); }
<YYINITIAL> "!="	{ return mkRet( SymbolTable.OP_NE ); }
<YYINITIAL> ";"		{ return mkRet( SymbolTable.SEMI ); }
<YYINITIAL> "{"		{ return mkRet( SymbolTable.LCURLY ); }
<YYINITIAL> "}"		{ return mkRet( SymbolTable.RCURLY ); }
<YYINITIAL> ","		{ return mkRet( SymbolTable.COMMA ); }
<YYINITIAL> ":"		{ return mkRet( SymbolTable.COLON ); }
<YYINITIAL> "="		{ return mkRet( SymbolTable.EQUALS ); }
<YYINITIAL> "("		{ return mkRet( SymbolTable.LPAREN ); }
<YYINITIAL> ")"		{ return mkRet( SymbolTable.RPAREN ); }
<YYINITIAL> "["		{ return mkRet( SymbolTable.LBRACK ); }
<YYINITIAL> "]"		{ return mkRet( SymbolTable.RBRACK ); }
<YYINITIAL> "."		{ return mkRet( SymbolTable.DOT ); }
<YYINITIAL> "&"		{ return mkRet( SymbolTable.BITAND ); }
<YYINITIAL> "!"		{ return mkRet( SymbolTable.NOT ); }
<YYINITIAL> "~"		{ return mkRet( SymbolTable.BITNOT ); }
<YYINITIAL> "-"		{ return mkRet( SymbolTable.MINUS ); }
<YYINITIAL> "+"		{ return mkRet( SymbolTable.PLUS ); }
<YYINITIAL> "*"		{ return mkRet( SymbolTable.TIMES ); }
<YYINITIAL> "/"		{ return mkRet( SymbolTable.DIVIDE ); }
<YYINITIAL> "%"		{ return mkRet( SymbolTable.MOD ); }
<YYINITIAL> "<"		{ return mkRet( SymbolTable.LESS ); }
<YYINITIAL> ">"		{ return mkRet( SymbolTable.GREATER ); }
<YYINITIAL> "^"		{ return mkRet( SymbolTable.XOR ); }
<YYINITIAL> "|"		{ return mkRet( SymbolTable.BITOR ); }
<YYINITIAL> "?"		{ return mkRet( SymbolTable.QUESTION ); }

<YYINITIAL> "[]"	{ return mkRet( SymbolTable.OP_EMPTYDIM ); }
<YYINITIAL> ".class"	{ return mkRet( SymbolTable.DOTCLASS ); }

<YYINITIAL> "*/"	{ error( E_ENDCOMMENT ); }
<YYINITIAL> .		{ error( E_UNMATCHED ); }

