/*
 * ArrayAccess.java 1.0
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>ArrayAccess</code> represents
 * an array access like :
 * <br><blockquote><pre>
 *     a[3][2]
 * </pre></blockquote><br>
 * In this array access expression,
 * you can get <code>a</code> by <code>getExpression()</code>
 * and can get <code>[3][2]</code> by <code>getDimExprList()</code> .
 *
 * @see openjava.ptree.Expression
 * @see openjava.ptree.DimExprList
 */
public class ArrayAccess extends NonLeaf
    implements Expression
{
  public ArrayAccess( Expression expr, DimExprList diml )
  {
    super();
    set( (Ptree) expr, diml );
  }

  ArrayAccess()
  {
    super();
  }
  
  public void writeCode()
  {
    writeDebugL();

    Expression expr = getExpression();
    if(expr instanceof Leaf
       || expr instanceof ArrayAccess
       || expr instanceof FieldAccess
       || expr instanceof MethodCall
       || expr instanceof QualifiedName){
      expr.writeCode();
    }else{
      out.print( "( " );
      expr.writeCode();
      out.print( " )" );
    }
    
    DimExprList diml = getDimExprList();
    diml.writeCode();

    writeDebugR();
  }

  /**
   * Gets the expression of array.
   *
   * @return  the experssion accessed as array.
   */
  public Expression getExpression()
  {
    return (Expression) elementAt( 0 );
  }

  /**
   * Sets the expression accessed as array.
   *
   * @param  expr  the experssion of array.
   */
  public void setExpression( Expression expr )
  {
    setElementAt( expr, 0 );
  }
  
  /**
   * Gets the dimexpr list.
   *
   * @return  the dimexpr list.
   */
  public DimExprList getDimExprList()
  {
    return (DimExprList) elementAt( 1 );
  }

  /**
   * Sets the dimexpr list.
   *
   * @param  dimexprs  the dimexpr list.
   */
  public void setDimExprList( DimExprList dimexprs )
  {
    setElementAt( dimexprs, 1 );
  }

}
