/*
 * ArrayInitializer.java 1.0
 * 
 * Jun 11, 1997 by mich
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


/**
 * The ArrayInitializer class presents initializer list of array elements.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.List
 * @see openjava.ptree.VariableInitializer
 */
public class ArrayInitializer extends List
    implements VariableInitializer
{
  /**
   * Allocates a new ArrayInitializer.
   *
   * @param arrayinit prototype object
   */
  public ArrayInitializer()
  {
    super( ", " );
  }

  public ArrayInitializer( VariableInitializer vi0 )
  {
    super( ", ", (Ptree) vi0 );
  }

  public ArrayInitializer(
      VariableInitializer vi0,
      VariableInitializer vi1 )
  {
    super( ", ", vi0, vi1 );
  }

  public void writeCode()
  {
    writeDebugL();

    out.print( "{ " );
    super.writeCode();
    out.print( " }" );

    writeDebugR();
  }

  /**
   * Gets the specified element at
   * This causes side-effect.
   *
   * @param  n  
   */
  public VariableInitializer elementAt( int n )
  {
    return (VariableInitializer) ptreelist.elementAt( n );
  }

  /**
   * Sets the specified element at the specified place of the list
   *
   * @param  p  element
   * @param  n  the number where to set element
   */
  public void setElementAt( VariableInitializer p, int n )
  {
    ptreelist.setElementAt( p, n );
  }

  /**
   * Adds the specified element after the list
   * This causes side-effect.
   *
   * @param  p  VariableInitializer to be inserted into the list
   */
  public void addElement( VariableInitializer p )
  {
    ptreelist.addElement( p );
  }

  /**
   * Inserts the specified element into the list
   * before the specified element of the list.
   * This causes side-effect.
   *
   * @param  p  the element to be inserted into the list
   * @param  n  number of the element before which insertion ocuurs
   */
  public void insertElementAt( VariableInitializer p, int n )
  {
    ptreelist.insertElementAt( p, n );
  }

  /**
   * Returns the first element
   *
   * @returns  the first element
   */
  public VariableInitializer getFirst()
  {
    return (VariableInitializer) ptreelist.getFirst();
  }

  /**
   * Returns the new list from this list's element except for
   * the first element
   *
   * @returns the rest list except for the first element
   */
  public ArrayInitializer getRest()
  {
    ArrayInitializer ret = new ArrayInitializer();
    for( int i = 1; i < getLength(); i++ ){
      ret.addElement( elementAt( i ) );
    }

    return ret;
  }

  public VariableInitializer lastElement()
  {
    return (VariableInitializer) ptreelist.lastElement();
  }

  /**
   * Appends a list after this list.
   *
   * @param  lst  a list to be appended
   */
  public void append( ArrayInitializer lst )
  {
    for( int i = 0; i < lst.getLength(); i++ ){
      ptreelist.addElement( lst.elementAt( i ) );
    }
  }


}


