/*
 * BreakStatement.java 1.0
 *
 *
 * Jun 20, 1997 mich
 * Sep 29, 1997 bv
 * Oct 11, 1997 mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 11, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>BreakStatement</code> class represents
 * a break statement node of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Statement
 * @see openjava.ptree.Identifier
 */
public class BreakStatement extends NonLeaf
    implements Statement
{
  /**
   * Allocates a new BreakStatement object.
   *
   * @param  label  the label of this break statemetn.
   *                if this is null, break statement has no label.
   */
  public BreakStatement( Identifier label )
  {
    super();
    set( label );
  }

  /**
   * Allocates a new BreakStatement object.
   *
   */
  public BreakStatement()
  {
    this( null );
  }
  
  public void writeCode()
  {
    writeTab();  writeDebugL();

    out.print( "break" );

    Identifier label = getLabel();
    if(label != null){
      out.print(" ");
      label.writeCode();
    }

    out.print( ";" );

    writeDebugR();  out.println();
  }
  
  /**
   * Gets the label of this break statement.
   *
   * @return  the label name.
   *          If there is no label then this method returns null.
   */
  public Identifier getLabel()
  {
    return (Identifier) elementAt( 0 );
  }

  /**
   * Sets the label of this break statement.
   *
   * @param  label  the label.
   */
  public void setLabel( Identifier label )
  {
    setElementAt( label, 0 );
  }


}
