/*
 * ClassDeclaration.java 1.0
 *
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The ClassDeclaration class presents class declaraton node
 * of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.TypeDeclaration
 */
public class ClassDeclaration extends NonLeaf
    implements TypeDeclaration
{
  private openjava.Class metaclazz = null;
  private UserDecoratorList userdecorator = null;

  /**
   * Constructs ClassDeclaration from its elements.
   *
   * @param modiflist modifier list
   * @param name class name
   * @param zuper arg zuper is null means class decl has no super class
   * @param interfacelist if class decl has no implemants,
   *                      arg interfacelist should be set an empty list
   * @param fieldlist field declaration list (body of new class)
   */
  public ClassDeclaration(
      ModifierList modiflist,
      Identifier name,
      ClassType zuper,
      ClassTypeList interfacelist,
      MemberDeclarationList fieldlist)
  {
    super();
    set( modiflist, name, zuper, interfacelist, fieldlist );
  }

  ClassDeclaration() {
    super();
  }

  /**
   * write object code
   *
   * ModifierList "class" Identifier
   *   "extends" ClassType "implements" ClassTypeList
   *   MemberDeclarationList
   */
  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null)  comment.writeCode();

    writeTab();  writeDebugL();

    //metaobject declaration
    openjava.Class clazz = getMetaclass();
    if(clazz != null){
      String mname = clazz.getClass().getName();
      if(! mname.equals( "openjava.Class" )){
	out.print( "metaobject " );
	Identifier name = getName();
	name.writeCode();
	out.print( " instanceof " + mname + ";" );
	writeTab();
      }
    }

    //ModifierList
    ModifierList modiflist = getModifiers();
    modiflist.writeCode();
    if(! modiflist.isEmpty()){
      out.print( " " );
    }
    
    //"class"
    out.print( "class " );
    
    Identifier name = getName();
    if(name != null)
      name.writeCode();
    
    //"extends" ClassType
    ClassType zuper = getSuper();
    if(zuper != null){
      out.print(" extends ");  zuper.writeCode();
    }else{
      writeDebug( " " );  writeDebugLR();
    }
    
    //"implements" ClassTypeList
    ClassTypeList implementz = getInterfaces();
    if(! implementz.isEmpty()){
      out.print(" implements ");  implementz.writeCode();
    }else{
      writeDebug( " " );  writeDebugLR();
    }

    out.println();
    
    //MemberDeclarationList
    MemberDeclarationList classbody = getBody();
    writeTab();  out.println( "{" );
    if(classbody.isEmpty()){
      classbody.writeCode();
    }else{
      out.println();
      pushNest();  classbody.writeCode();  popNest();
      out.println();
    }
    writeTab();  out.print( "}" );

    writeDebugR();  out.println();
  }
  
  /**
   * Gets modifier list
   *
   * @return there is no modifiers, getModifierList returns an empty list.
   */
  public ModifierList getModifiers()
  {
    return (ModifierList) elementAt( 0 );
  }

  /**
   * Sets modifier list
   *
   * @param  modifs  modifiers to set
   */
  public void setModifiers( ModifierList modifs )
  {
    setElementAt( modifs, 0 );
  }
  
  /**
   * Gets class name
   *
   * @return class name
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets class name
   *
   * @param  name  name to set
   */
  public void setName( Identifier name )
  {
    setElementAt( name, 1 );
  }
  
  /**
   * Gets super class name
   *
   * @return if class decl has no extends, this returns null
   *         otherwise returns the name of the super class.
   */
  public ClassType getSuper()
  {
    return (ClassType) elementAt( 2 );
  }

  /**
   * Sets super class name
   *
   * @param  ct  class type to set
   */
  public void setSuper( ClassType ct )
  {
    setElementAt( ct, 2 );
  }
  
  /**
   * Gets interface name list
   * 
   * @return there is no implemented class, getInterfaceList returns
   *         an empty list
   */
  public ClassTypeList getInterfaces()
  {
    return (ClassTypeList) elementAt( 3 );
  }

  /**
   * Sets interface name list
   * 
   * @param  ctlist  class type list to set
   */
  public void setInterfaces( ClassTypeList ctlist )
  {
   setElementAt( ctlist, 3 );
  }
  
  /**
   * Gets class body
   *
   * @return return an field declaration list as this class body.
   */
  public MemberDeclarationList getBody()
  {
    return (MemberDeclarationList) elementAt( 4 );
  }

  /**
   * Sets class body
   *
   * @param  body  member declaration list to set as this class body.
   */
  public void setBody( MemberDeclarationList mdlist )
  {
    setElementAt( mdlist, 4 );
  }

  public UserDecoratorList getUserDecorators()
  {
    return this.userdecorator;
  }

  public void setUserDecorators( UserDecoratorList ud )
  {
    if(ud == null)  ud = new UserDecoratorList();
    this.userdecorator = ud;
  }

  public void setMetaclass( openjava.Class metaclazz )
  {
    this.metaclazz = metaclazz;
  }

  public openjava.Class getMetaclass()
  {
    return this.metaclazz;
  }

}
