/*
 * ClassLiteral.java 1.0
 *
 *
 * Jun 20, 1997
 * Oct 10, 1997
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Sep 29, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>ClassLiteral</code> class represents
 * an expression as a object of <code>Class</code> class,
 * which is suppoted since JDK 1.1.
 * This is like :
 * <br><blockquote><pre>
 *     String.class
 * </pre></blockquote><br>
 * or :
 * <br><blockquote><pre>
 *     int.class
 * </pre></blockquote><br>
 * 
 * @see java.lang.Class
 * @see openjava.ptree.Leaf
 * @see openjava.ptree.Expression
 * @see openjava.ptree.TypeName
 */
public class ClassLiteral extends Leaf
    implements Expression
{

  TypeName tn = null;

  /**
   * Allocates a new object.
   *
   */
  public ClassLiteral( TypeName tn )
  {
    super( tn.toString() + ".class" );
    this.tn = tn;
  }

  /**
   * Gets the type name of this class literal.
   *
   * @return  the type name.
   */
  public TypeName getTypeName()
  {
    return tn;
  }

}
