/*
 * ClassType.java 1.0
 *
 *
 * Jun 20, 1997 by mich
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import openjava.main.SymbolTable;


/**
 * The <code>ClassType</code> class presents for a class type
 * specificator.
 * The class type specificator consists of a package name and
 * a short class name.
 * <br>
 * If this object presents for:
 * <br><blockquote><pre>
 *     java.lang.String
 * </pre></blockquote><br>
 * The package name is <code>java.lang</code>, and
 * the short class name is <code>String</code>.
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.TypeName
 * @see openjava.ptree.QualifiedName
 * @see openjava.ptree.Identifier
 */
public class ClassType extends NonLeaf
    implements TypeName
{
  /**
   * Allocates new object.
   *
   * @param  pack  package name of class
   * @param  name  short class name
   */
  public ClassType(
    QualifiedName  pack,
    Identifier     name )
  {
    super();
    if(pack == null){
      pack = new QualifiedName();
    }
    set( pack, name );
  }

  ClassType()
  {
    super();
  }

  /**
   * Allocates new object.
   *
   * @param  name  short class name
   */
  public ClassType(
    Identifier     name )
  {
    this( new QualifiedName(), name );
  }
  
  public void writeCode()
  {
    writeDebugL();

    QualifiedName qn = getPackage();
    if(! qn.isEmpty()){
      qn.writeCode();
      out.print( "." );
    }

    Identifier name = getName();
    name.writeCode();

    writeDebugR();
  }

  /**
   * Gets the package name of this class type specificator.
   *
   * @return  the package name of this class type specificator
   */
  public QualifiedName getPackage()
  {
    return (QualifiedName) elementAt( 0 );
  }

  /**
   * Sets the package name of this class type specificator.
   *
   * @param  packname  the package name of this class type specificator.
   *                   to set
   */
  public void setPackage( QualifiedName packname )
  {
    setElementAt( packname, 0 );
  }
  
  /**
   * Gets the short class name of this class type specificator.
   *
   * @return  the short class name of this class type specificator.
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets the short class name of this class type specificator.
   *
   * @param  name  the short class name of this class type specificator.
   *               to set
   */
  public void setName( Identifier name )
  {
    setElementAt( name, 1 );
  }

}
