/*
 * ContinueStatement.java 1.0
 *
 *
 * Jun 20, 1997 mich
 * Sep 29, 1997 bv
 * Oct 11, 1997 mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 11, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>ContinueStatement</code> class represents
 * a continue statement node of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Statement
 * @see openjava.ptree.Identifier
 */
public class ContinueStatement extends NonLeaf
    implements Statement
{
  /**
   * Allocates a new ContinueStatement object.
   *
   * @param  label  the label name.
   */
  public ContinueStatement( Identifier label )
  {
    super();
    set( label );
  }

  /**
   * Allocates a new ContinueStatement object.
   *
   */
  public ContinueStatement()
  {
    this( null );
  }
  
  public void writeCode()
  {
    writeTab();  writeDebugL();

    out.print("continue");

    Identifier label = getLabel();
    if(label != null){
      out.print(" ");
      label.writeCode();
    }

    out.print( ";" );

    writeDebugR();  out.println();
  }
  
  /**
   * Gets the label of this break statement.
   *
   * @return  the label name.
   *          If there is no label then this method returns null.
   */
  public Identifier getLabel()
  {
    return (Identifier) elementAt( 0 );
  }

  /**
   * Sets the label of this break statement.
   *
   * @param  label  the label.
   */
  public void setLabel( Identifier label )
  {
    setElementAt( label, 0 );
  }

}
