/*
 * FieldAccess.java 1.0
 *
 * Jun 20, 1997
 * Sep 29, 1997
 * Oct 10, 1997
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>FieldAccess</code> class represents
 * a field access like :
 * <br><blockquote><pre>
 *     f().str
 * </pre></blockquote><br>
 * In this field access,
 * you can get <code>f()</code> by <code>getExpression()</code>
 * and can get <code>str</code> by <code>getName()</code> .
 * <br>
 * Warning this class may has bugs around the expression.
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Expression
 * @see openjava.ptree.Identifier
 */
public class FieldAccess extends NonLeaf
    implements Expression
{
  public FieldAccess( Expression expr, Identifier name )
  {
    super();
    set( (Ptree) expr, name );
  }

  FieldAccess()
  {
    super();
  }

  public void writeCode()
  {
    writeDebugL();

    Expression expr = getExpression();
    if(expr instanceof Leaf
       || expr instanceof ArrayAccess
       || expr instanceof FieldAccess
       || expr instanceof MethodCall
       || expr instanceof QualifiedName){
      expr.writeCode();
    }else{
      out.print( "(" );
      expr.writeCode();
      out.print( ")" );
    }

    out.print( "." );
    
    Identifier name = getName();
    name.writeCode();

    writeDebugR();
  }

  /**
   * Gets the expression accessed.
   *
   * @return  the expression accessed.
   */
  public Expression getExpression()
  {
    return (Expression) elementAt( 0 );
  }

  /**
   * Sets the expression accessed.
   *
   * @param  expr  the expression accessed.
   */
  public void setExpression( Expression expr )
  {
    setElementAt( expr, 0 );
  }

  /**
   * Gets the field name.
   *
   * @return  the field name.
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets the field name.
   *
   * @param  name  the field name.
   */
  public void setName( Identifier name )
  {
    setElementAt( name, 1 );
  }

}
