/*
 * FieldDeclaration.java 1.0
 *
 * Jun 11, 1997
 *  
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated: Sep 4, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import openjava.util.PartialParser;


/**
 * The FieldDeclaration class presents for node of parse tree.
 * FieldDeclaration
 *   :=  ModifierList TypeSpecifier VariableDeclarator SEMI
 *
 */
public class FieldDeclaration extends NonLeaf
    implements MemberDeclaration
{
  /**
   * Allocates this object
   *
   */
  public FieldDeclaration(
      ModifierList        e0,
      TypeSpecifier       e1,
      VariableDeclarator  e2 )
  {
    super();
    set( e0, e1, e2 );
  }

  public FieldDeclaration(
      ModifierList         e0,
      TypeSpecifier        e1,
      Identifier           e2,
      VariableInitializer  e3 )
  {
    super();
    VariableDeclarator vd = new VariableDeclarator( e2, e3 );
    set( e0, e1, vd );
  }

  FieldDeclaration()
  {
    super();
  }

  /**
   * Overrides writing code method
   *
   */
  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null)  comment.writeCode();

    writeTab();
    writeDebugL();

    //ModifierList
    ModifierList modiflist = getModifiers();
    modiflist.writeCode();
    if(! modiflist.isEmpty()){
      out.print( " " );
    }

    //TypeSpecifier
    TypeSpecifier ts = getTypeSpecifier();
    ts.writeCode();

    out.print( " " );

    //VariableDeclarator
    VariableDeclarator vd = getVariableDeclarator();
    vd.writeCode();

    //";"
    out.print( ";" );

    writeDebugR();
    out.println();
  }

  /**
   * Gets modifier list of this field.
   *
   * @return  modifier list. if there is no modifiers, this returns
   *          an empty list.
   */
  public ModifierList getModifiers()
  {
    return (ModifierList) elementAt( 0 );
  }

  /**
   * Sets modifier list of this field.
   *
   * @param  modifs  modifiers to set
   */
  public void setModifiers( ModifierList modifs )
  {
    setElementAt( modifs, 0 );
  }

  /**
   * Gets type specifier of this field variable.
   *
   * @return  modifier list
   */
  public TypeSpecifier getTypeSpecifier()
  {
    return (TypeSpecifier) elementAt( 1 );
  }

  /**
   * Sets type specifier of this field variable.
   *
   * @param  tspec  type specifier to set
   */
  public void setTypeSpecifier( TypeSpecifier tspec )
  {
    setElementAt( tspec, 1 );
  }

  /**
   * Gets variable declarator of this field
   *
   * @return  variable declarator
   * @see openjava.tree.VariableDeclarator
   */
  public VariableDeclarator getVariableDeclarator()
  {
    return (VariableDeclarator) elementAt( 2 );
  }

  /**
   * Sets type specifier of this field variable.
   *
   * @param  vdeclr  variable declarator to set
   */
  public void setVariableDeclarator( VariableDeclarator vdeclr )
  {
    setElementAt( vdeclr, 2 );
  }

  /**
   * Gets variable name of this field.
   *
   * @return  identifier of field variable
   */
  public Identifier getVariable()
  {
    return (Identifier) getVariableDeclarator().getVariable();
  }

  /**
   * Sets variable name of this field.
   *
   * @return  identifier of field variable
   */
  public void setVariable( Identifier name )
  {
    getVariableDeclarator().setVariable( name );
  }

  /**
   * Gets variable initializer of this field.
   *
   * @return  variable initializer
   */
  public VariableInitializer getInitializer()
  {
    return (VariableInitializer) getVariableDeclarator().getInitializer();
  }

  /**
   * Gets variable initializer of this field.
   *
   * @return  variable initializer
   */
  public void setInitializer( VariableInitializer vinit )
  {
    getVariableDeclarator().setInitializer( vinit );
  }

  public static FieldDeclaration make( String str )
  {
    FieldDeclaration ret = null;

    try{
      ret = (FieldDeclaration) PartialParser.makeMemberDeclaration( str );
    }catch( Exception ex ){
      return null;
    }

    return ret;
  }

}
