/*
 * ImportStatement.java 1.0
 *
 * This subclass of symbol represents (at least) terminal symbols returned 
 * by the scanner and placed on the parse stack.  At present, this 
 * class does nothing more than its super class.
 *
 * Jun 11, 1997 by mich
 *  
 * @see openjava.ptree.Ptree
 * @version  1.0 last updated: Sep 27, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.ptree.*;


/**
 * The ImportStatement class presents for import statement of compilation
 * unit.
 *
 * This consists of
 *          (path)            (classname or *)
 * "import" QualifiedName "." Identifier       ";"
 *
 * If identifier is null, it presents for a multiple classes importation.
 *
 */
public class ImportStatement extends NonLeaf
{
  /**
   * Allocates this object with specified parse-tree elements.
   *
   */
  public ImportStatement(
      QualifiedName e0,
      Identifier e1 )
  {
    super();
    set( e0, e1 );
  }

  /**
   * Overrides writing code method
   *
   */
  public void writeCode()
  {
    writeDebugL();

    //"import"
    out.print( "import " );

    //qualifiedName
    {
      QualifiedName qn = (QualifiedName) elementAt( 0 );
      if(qn != null){
        qn.writeCode();
      }
    }

    //"."
    out.print( "." );
    
    //"*" or identifier
    {
      Identifier ident = (Identifier) elementAt( 1 );
      if(ident == null){
	out.print( "*" );
      }else{
	ident.writeCode();
      }
    }

    //";"
    out.print( ";" );

    writeDebugR();
  }

  public void setPath( QualifiedName qn )
  {
    setElementAt( qn, 0 );
  }
  public QualifiedName getPath()
  {
    return (QualifiedName) elementAt( 0 );
  }
  
  public void setClassName( Identifier ident )
  {
    setElementAt( ident, 1 );
  }
  public Identifier getClassName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * NOT IMPLEMENTED
   *
   */
  public void setFull( QualifiedName qn )
  {
    ;
  }

  /**
   * Tests if this import statement includes multiple classes importing
   * like "java.lang.*;".
   *
   * @return  true  if this imports multiple classes 
   */
  public boolean isMulti()
  {
    return (elementAt( 1 ) == null);
  }

}
