/*
 * InstanceInitilizer.java 1.0
 *
 * Jun 20, 1997
 * Sep 29, 1997
 * Oct 10, 1997
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The InstanceInitilizer class represents instance initializer block
 * of parse tree.
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.MemberDeclaration
 * @see openjava.ptree.StatementList
 */
public class InstanceInitializer extends NonLeaf
    implements MemberDeclaration
{
  /**
   * Allocates a new object.
   *
   * @param  stmts  the statement list of this instance initializer block
   */
  public InstanceInitializer( StatementList block )
  {
    super();
    set( block );
  }

  InstanceInitializer()
  {
    super();
  }
  
  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null)  comment.writeCode();

    writeTab();  writeDebugL();

    out.println( "{" );

    StatementList block = getBody();
    if(block != null){
      pushNest();  block.writeCode();  popNest();
    }

    writeTab();  out.println( "}" );  writeDebugR();
  }
  
  /**
   * Gets the body of this instance initializer.
   *
   * @return  statement list.
   */
  public StatementList getBody()
  {
    return (StatementList) elementAt( 0 );
  }

  /**
   * Sets the body of this instance initializer.
   *
   * @param  stmts  statement list to set.
   */
  public void setBody( StatementList stmts )
  {
    setElementAt( stmts, 0 );
  }

}
