/*
 * InterfaceDeclaration.java 1.0
 *
 * This interface is made to type ptree-node into the inteface
 * declaration in the body of class.
 *
 * Jun 20, 1997 by mich
 * Aug 20, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The InterfaceDeclaration class presents Interface declaration node
 * of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.TypeDeclaration
 */
public class InterfaceDeclaration extends NonLeaf
    implements TypeDeclaration
{
  private openjava.Class metaclazz = null;
  private UserDecoratorList userdecorator = null;

  /**
   * Constructs InterfaceDeclaration from its elements
   * Allocates new object, and assigns the specified objects
   * to its instance variables.
   *
   * @param modiflist modifier words of the new interface
   * @param name the name of the new interface
   * @param extendlist extends interfaces of the new interface
   * @param fieldlist field declarations, it is body of the new interface
   */
  public InterfaceDeclaration(
    ModifierList modiflist,
    Leaf name,
    ClassTypeList extendlist,
    MemberDeclarationList fieldlist )
  {
    super();
    set( modiflist, name, extendlist, fieldlist );
  }

  public InterfaceDeclaration() {
    super();
  }
  
  /**
   * write object code
   *
   * ModifierList "interface" Identifier
   *   "extends" ClassTypeList MemberDeclarationList
   */
  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null)  comment.writeCode();

    writeTab();  writeDebugL();

    //metaobject declaration
    openjava.Class clazz = getMetaclass();
    if(clazz != null){
      String mname = clazz.getClass().getName();
      if(! mname.equals( "openjava.Class" )){
        out.print( "metaobject " );
        Identifier name = getName();
        name.writeCode();
        out.print( " instanceof " + mname + ";" );
        writeTab();
      }
    }

    ModifierList modiflist = getModifiers();
    modiflist.writeCode();
    if(! modiflist.isEmpty()){
      out.print( " " );
    }
    
    out.print( "interface " );
    
    Identifier name = getName();
    name.writeCode();
    
    ClassTypeList extendz = getSupers();
    if(! extendz.isEmpty()){
      out.print(" extends ");
      extendz.writeCode();
    }

    out.println();

    MemberDeclarationList interfacebody = getBody();
    writeTab();  out.println( "{" );
    if(interfacebody.isEmpty()){
      interfacebody.writeCode();
    }else{
      out.println();
      pushNest();  interfacebody.writeCode();  popNest();
      out.println();
    }      
    writeTab();  out.print( "}" );

    writeDebugR();  out.println();
  }

  /**
   * Gets modifier list
   *
   * @return if there is no modifiers,
   *         getModifiers() returns an empty list,
   *         otherwise returns modifier list.
   */
  public ModifierList getModifiers()
  {
    return (ModifierList) elementAt( 0 );
  }

  /**
   * Sets modifier list
   *
   * @param  modifs  modifier list to set
   */
  public void setModifiers( ModifierList modifs )
  {
    setElementAt( modifs, 0 );
  }
  
  /**
   * Gets interface name
   *
   * @return interface name
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets interface name
   *
   * @param  name  interface name to set
   */
  public void setName( Identifier name )
  {
   setElementAt( name, 1 );
  }
  
  /**
   * Gets extends name list
   * 
   * @return there is no extends, getExtendsList returns an empty list
   */
  public ClassTypeList getSupers()
  {
    return (ClassTypeList) elementAt( 2 );
  }

  /**
   * Sets extends name list
   * 
   * @param  zupers  class type list to set
   */
  public void setSupers( ClassTypeList zupers )
  {
    setElementAt( zupers, 2 );
  }
  
  /**
   * Gets interface body
   *
   * @return an field declaration list as the interface body.
   */
  public MemberDeclarationList getBody()
  {
    return (MemberDeclarationList) elementAt( 3 );
  }

  /**
   * Gets interface body
   *
   * @param  body  member declaration list to set
   */
  public void setBody( MemberDeclarationList body )
  {
    setElementAt( body, 3 );
  }

  public void setUserDecorators( UserDecoratorList ud )
  {
    if(ud == null)  ud = new UserDecoratorList();
    this.userdecorator = ud;
  }

  public UserDecoratorList getUserDecorators()
  {
    return this.userdecorator;
  }

  public void setMetaclass( openjava.Class metaclazz )
  {
    this.metaclazz = metaclazz;
  }

  public openjava.Class getMetaclass()
  {
    return this.metaclazz;
  }

}
