/*
 * List.java 1.0
 * 
 * Jun 11, 1997 by mich
 * Aug 20, 1997 by mich
 * Sep 28, 1997 by mich
 * 
 * @see java_cup.runtime.symbol
 * @version 1.0 last updated: Sep 28, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.PrintStream;
import openjava.tools.UtilityList;


/**
 * The List class presents for the list of parse trees.
 *
 */
public abstract class List extends PtreeObject
implements Ptree
{

  /** The ptreelist  the list of parse-tree nodes */
  protected UtilityList ptreelist = null;

  /** The ptreelist  the list of parse-tree nodes */
  private String delimiter = PtreeObject.LN;

  /**
   * Allocates this List
   *
   * @param p0  list's element
   */
  protected List()
  {
    ptreelist = new UtilityList();
  }

  /**
   * Allocates this List
   *
   * @param p0  list's element
   */
  protected List( Ptree p )
  {
    this();
    ptreelist.setList( p );
  }

  /**
   * Allocates this List
   *
   */
  protected List( String delimiter )
  {
    this();
    this.delimiter = delimiter;
  }

  /**
   * Allocates this List
   *
   * @param p0  list's element
   */
  protected List( String delimiter, Ptree p )
  {
    this( delimiter );
    ptreelist.setList( p );
  }

  /**
   * Allocates this List with the specified ptree elements.
   *
   * @param p0  list's element
   * @param p1  list's element
   */
  protected List( String delimiter, Ptree p0, Ptree p1 )
  {
    this( delimiter );
    ptreelist.setList( p0, p1 );
  }

  /**
   * Allocates this List with the specified ptree elements.
   *
   * @param p0  list's element
   * @param p1  list's element
   * @param p2  list's element
   */
  protected List( String delimiter, Ptree p0, Ptree p1, Ptree p2 )
  {
    this( delimiter );
    ptreelist.setList( p0, p1, p2 );
  }

  /**
   * Allocates this List with the specified ptree elements.
   *
   * @param p0  list's element
   * @param p1  list's element
   * @param p2  list's element
   * @param p3  list's element
   */
  protected List( String delimiter, Ptree p0, Ptree p1, Ptree p2, Ptree p3 )
  {
    this( delimiter );
    ptreelist.setList( p0, p1, p2, p3 );
  }

  protected void set( UtilityList lst )
  {
    this.ptreelist = lst;
  }

  /**
   * Returns the length of this list.
   *
   * @return  the length of this list
   */
  public int getLength()
  {
    return ptreelist.getLength();
  }

  /**
   * Tests if this list is empty.
   *
   * @return  true if this list is empty
   */
  public boolean isEmpty()
  {
    return ptreelist.isEmpty();
  }

  /**
   * Removes an element at the specified point from this list
   *
   * @param  n  where to remove element.
   */
  public void removeElementAt( int n )
  {
    ptreelist.removeElementAt( n );
  }

  /**
   * Removes the last element from this list
   * This equals to  obj.removeElemntAt( obj.getLength() )
   *
   * @param  n  where to remove element.
   */
  public void removeLastElement()
  {
    ptreelist.removeElementAt( size() - 1 );
  }

  public int size()
  {
    return ptreelist.size();
  }

  /**
   * Removes an element at the specified point from this list
   *
   * @param  n  where to remove element.
   */
  public void removeAllElements()
  {
    ptreelist.removeAllElements();
  }

  /**
   * Tests if this list-node's value equals to the specified
   * ptree-node's.
   *
   * @return  true if two values are same.
   */
  public boolean equals( Ptree p )
  {
    if(p == null)  return false;
    if(eq( p ))  return true;
    if(this.getClass() != p.getClass())  return false;

    List nlp = (List) p;
    int len = this.getLength();
    if(len != nlp.getLength())  return false;
    for( int i = 0; i < len; i++ ){
      if(! equal( (Ptree) ptreelist.elementAt( i ),
		   (Ptree) nlp.ptreelist.elementAt( i ) )){
        return false;
      }
    }
    return true;
  }

  /**
   * Makes a new copy (another object) of this list-node.
   * The objects contained by this object will also be copied
   *
   * @return  the copy of this nonleaf-node as a ptree-node.
   */
  public Ptree makeRecursiveCopy()
  {
    UtilityList newlist = new UtilityList();
    
    for( int i = 0; i < getLength(); i++ ){
      Ptree elem = (Ptree) ptreelist.elementAt( i );
      if(elem != null)  elem = elem.makeRecursiveCopy();
      newlist.addElement( elem );
    }

    List ret = null;
    try{
      ret = (List) this.getClass().newInstance();
    }catch( InstantiationException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }catch( IllegalAccessException ex ){
      System.err.println( "makeCopy() error : " + ex );
      return null;
    }
    ret.set( newlist );

    return ret;
  }

  /**
   * Makes a new copy (another object) of this list-node.
   * The objects contained by the new object are same as
   * these contained by the original object.
   *
   * @return  the copy of this nonleaf-node as a ptree-node.
   */
  public Ptree makeCopy()
  {
    UtilityList newlist = new UtilityList();
    
    for( int i = 0; i < getLength(); i++ ){
      Ptree elem = (Ptree) ptreelist.elementAt( i );
      newlist.addElement( elem );
    }

    List ret = null;
    try{
      ret = (List) this.getClass().newInstance();
    }catch( InstantiationException ex ){
      System.err.println( "makeCopy() error : " + ex );
    }catch( IllegalAccessException ex ){
      System.err.println( "makeCopy() error : " + ex );
    }
    ret.set( newlist );

    return ret;
  }

  /**
   * Writes the code this parse-tree presents for.
   *
   */
  public void writeCode()
  {

    if(isEmpty())  return;

    Ptree pelem = (Ptree) ptreelist.elementAt( 0 );
    if(pelem == null){
      writeDebug( "#" );
    }else{
      pelem.writeCode();
    }
    for( int i = 1; i < getLength(); i++ ){
      out.print( this.delimiter );
      Ptree p = (Ptree) ptreelist.elementAt( i );
      if(p == null){
	writeDebug( "#" );
      }else{
	p.writeCode();
      }
    }
  }

  /**
   * Tests if any element in this list equal to the specified string.
   * No NonLeaf object will not matched with any strings.
   *
   * @param  str  the string to compare.
   * @return  true  if one or more elements who represent the specified
   *                string is find in this list.
   * @see openjava.ptree.Leaf#equals(String)
   */
  public boolean contains( String str )
  {
    int len = getLength();
    for( int i = 0; i < len; i++ ){
      Ptree p = (Ptree) ptreelist.elementAt( i );
      if(p == null){
	if(str == null || str.equals( "" ))  return true;
	continue;
      }
      if(! (p instanceof Leaf))  continue;
      Leaf l = (Leaf) p;
      if(l.equals( str ))  return true;
    }
    return false;
  }

  /**
   * Tests if any element in this list equal to the specified string.
   * No NonLeaf object will not matched with any strings.
   *
   * @param  str  the string to compare.
   * @return  true  if one or more elements who represent the specified
   *                string is find in this list.
   * @see openjava.ptree.Ptree#equals(Ptree)
   */
  public boolean contains( Ptree q )
  {
    int len = getLength();
    for( int i = 0; i < len; i++ ){
      Ptree p = (Ptree) ptreelist.elementAt( i );
      if(p == null){
	if(q == null)  return true;
	continue;
      }
      if(p.equals( q ))  return true;
    }
    return false;
  }

}
