/*
 * Literal.java 1.0
 *
 * Jun 20, 1997
 * Sep 29, 1997
 * Oct 10, 1997
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>Literal class</code> represents
 * a literal.
 *
 * @see openjava.ptree.Leaf
 * @see openjava.ptree.Expression
 */
public class Literal extends Leaf
    implements Expression
{
  public static final int TRUE = 0;
  public static final int FALSE = 1;
  public static final int STRING = 2;
  public static final int NUMBER = 3;
  public static final int CHARACTER =  4;
  static final String[] literal_name
  = { "true", "false", "\"\"" };
  
  int id = -1;

  /**
   * Allocates a new object.
   *
   * @param  id  the id number of the literal.
   */
  public Literal( int id )
  {
    super( id2str( id ) );
    this.id = id;
  }

  /**
   * Allocates a new object.
   * If you want to make <code>String</code> literal like
   * <code>"test"</code>,
   * call this constructor in the form :
   * <br><blockquote><pre>
   *     new Literal( Literal.STRING, "\"test\"" )
   * </pre></blockquote><br>
   * or use makeLiteral() static method. 
   *
   * @param  id  the id number of the literal.
   * @param  str  the literal as a String.
   * @see openjava.ptree.Literal#makeLiteral(String)
   */  
  public Literal( int id, String str )
  {
    super( str );
    this.id = id;
  }

    /**
     * Returns the id number indicating the kind of this literal.
     *
     * @return  the id number
     */
    public int getKind() {
	return this.id;
    }

    /**
     * Use getKind()
     * @deprecated
     */
    public int getID() { return getKind(); }
  
  private static String id2str( int id )
  {
    if(id == TRUE || id == FALSE || id == STRING ){
      return literal_name[id];
    }

    return null;
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the string.
   *
   * @param  str  the string.
   */
  public static Literal makeLiteral( String str )
  {
    return new Literal( Literal.STRING, "\"" + str + "\"" );
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the boolean.
   *
   * @param  b  the boolean.
   */
  public static Literal makeLiteral( boolean b )
  {
    if(b){
      return new Literal( Literal.TRUE, "true" );
    }else{
      return new Literal( Literal.FALSE, "false" );
    }
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the character.
   *
   * @param  c  the character.
   */
  public static Literal makeLiteral( char c )
  {
    return new Literal( Literal.NUMBER, "'" + c  + "'"  );
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the number.
   *
   * @param  num  the number.
   */
  public static Literal makeLiteral( int num )
  {
    return new Literal( Literal.NUMBER, String.valueOf( num ) );
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the number.
   *
   * @param  num  the number.
   */
  public static Literal makeLiteral( long num )
  {
    return new Literal( Literal.NUMBER, String.valueOf( num ) + "l" );
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the number.
   *
   * @param  num  the number.
   */
  public static Literal makeLiteral( float num )
  {
    return new Literal( Literal.NUMBER, String.valueOf( num ) + "f" );
  }

  /**
   * Makes a new object of <code>Literal</code> class
   * from the number.
   *
   * @param  num  the number.
   */
  public static Literal makeLiteral( double num )
  {
    return new Literal( Literal.NUMBER, String.valueOf( num ) + "d" );
  }


}
