/*
 * MethodCall.java 1.0
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>MethodCall</code> class represents
 * a method call expression.
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Expression
 * @see openjava.ptree.Identifier
 */
public class MethodCall extends NonLeaf
    implements Expression
{
  /**
   * Allocates a new object.
   *
   * @param  expr  the expression which indicates an object or
   *               a class. This may be null.
   * @param  name  the method name.
   * @param  args  the argumetns for this method.
   */
  public MethodCall(
      Expression      expr,
      Identifier      name,
      ExpressionList  args )
  {
    super();
    set( (Ptree) expr, name, args );
  }

  /**
   * Allocates a new object.
   *
   * @param  name  the method name.
   * @param  args  the argumetns for this method.
   */
  public MethodCall(
      Identifier      name,
      ExpressionList  args )
  {
    this( null, name, args );
  }

  MethodCall() {
    super();
  }

  public void writeCode()
  {
    writeDebugL();

    Expression expr = getExpression();

    if(expr != null){
      if(expr instanceof Leaf
	 || expr instanceof ArrayAccess
	 || expr instanceof FieldAccess
	 || expr instanceof MethodCall
	 || expr instanceof QualifiedName){
	expr.writeCode();
      }else{
	out.print( "( " );
	expr.writeCode();
	out.print( " )" );
      }
      out.print( "." );
    }

    Identifier name = getName();
    name.writeCode();

    out.print( "(" );
    ExpressionList args = getArguments();
    if(args.isEmpty()){
      args.writeCode();
    }else{
      out.print( " " );
      args.writeCode();
      out.print( " " );
    }
    out.print( ")" );

    writeDebugR();
  }
  
  /**
   * Gets the expression accessed.
   *
   * @return  the expression accessed.
   */
  public Expression getExpression()
  {
    return (Expression) elementAt( 0 );
  }

  /**
   * Sets the expression accessed.
   *
   * @param  expr  the expression accessed.
   */
  public void setExpression( Expression expr )
  {
    setElementAt( expr, 0 );
  }

  /**
   * Gets the method name.
   *
   * @return  the method name.
   */
  public Identifier getName()
  {
    return (Identifier) elementAt( 1 );
  }

  /**
   * Sets the method name.
   *
   * @param  name  the method name.
   */
  public void setName( Identifier name )
  {
    setElementAt( name, 1 );
  }

  /**
   * Gets the arguments for this method.
   *
   * @return  the expression list as the arguments.
   */
  public ExpressionList getArguments()
  {
    return (ExpressionList) elementAt( 2 );
  }

  /**
   * Sets the arguments for this method.
   *
   * @param  exprs  the expression list as the arguments.
   */
  public void setArguments( ExpressionList exprs )
  {
    setElementAt( exprs, 2 );
  }

}
