/*
 * Parameter.java 1.0
 *
 * This interface is made to type ptree-node into
 * the parameter in the body of class.
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Sep 29, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The Parameter class represents parameter node of parse tree.
 * Modifiers of parameter are supported from JDK 1.1.
 * The code like:
 * <br><blockquote><pre>
 *     void test( final int i ){
 *         ....
 *     }
 * </pre></blockquote><br>
 * is allowed from JDK 1.1.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.ModifierList
 * @see openjava.ptree.TypeSpecifier
 * @see openjava.ptree.Identifier
 */
public class Parameter extends NonLeaf
  {
  /**
   * Allocates a new object.
   *
   * @param  modfiers  modifier list of the new parameter
   * @param  typespec  type specifier includes array dimension info
   * @param  declname  the parameter's name, including no array dim.
   */
  public Parameter(
    ModifierList modiflist,
    TypeSpecifier typespec,
    Identifier declname )
  {
    super();
    if(modiflist == null){
      modiflist = new ModifierList();
    }
    set( modiflist, typespec, declname );
  }

  Parameter() {
    super();
  }

  /**
   * Allocates a new object.
   * 
   *
   * @param typespec type specifier includes array dimension info
   * @param declname the parameter's name, also includes array dim
   *        arg modfier is null means parameter has no modifier
   */
  public Parameter(
    TypeSpecifier typespec,
    Identifier declname )
  {
    this( new ModifierList(), typespec, declname );
  }
  
  public void writeCode()
  {
    writeDebugL();

    ModifierList modifs = getModifiers();
    modifs.writeCode();
    if(! modifs.isEmpty()){
      out.print( " " );
    }
    
    TypeSpecifier typespec = getTypeSpecifier();
    typespec.writeCode();

    out.print( " " );
    
    Identifier declname = getVariable();
    declname.writeCode();

    writeDebugR();
  }
  
  /**
   * Gets the modifiers of this parameter.
   *
   * @return the modfiers.
   */
  public ModifierList getModifiers()
  {
    return (ModifierList) elementAt( 0 );
  }

  /**
   * Sets the modifiers of this parameter.
   *
   * @param  modifs  the modfiers to set.
   */
  public void setModifiers( ModifierList modifs )
  {
    setElementAt( modifs, 0 );
  }
  
  /**
   * Gets the type specifier of this parameter.
   *
   * @return the type specifier.
   */
  public TypeSpecifier getTypeSpecifier()
  {
    return (TypeSpecifier) elementAt( 1 );
  }

  /**
   * Sets the type specifier of this parameter.
   *
   * @param  tspec  the type specifier to set.
   */
  public void setTypeSpecifier( TypeSpecifier tspec )
  {
    setElementAt( tspec, 1 );
  }
  
  /**
   * Gets the variable name of this parameter.
   *
   * @return the variable name.
   */
  public Identifier getVariable()
  {
    return (Identifier) elementAt( 2 );
  }

  /**
   * Sets the variable name of this parameter.
   *
   * @param  varname  the variable name to set.
   */
  public void setVariable( Identifier varname )
  {
    setElementAt( varname, 2 );
  }

}
