/*
 * QualifiedName.java 1.0
 *  
 * Jun 11, 1997
 *
 * @see openjava.ptree.Ptree
 * @version last updated: 06/11/97
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.util.PartialParser;
import openjava.tools.*;


/**
 * The QualifiedName class presents for the node of parse tree
 * of qualified name.
 *
 */
public class QualifiedName extends List
    implements Expression
{
  public QualifiedName() {
    super( "." );
  }

  public QualifiedName( Identifier e0 )
  {
    super( ".", e0 );
  }

  public QualifiedName( Identifier e0, Identifier e1 )
  {
    super( ".", e0, e1 );
  }

  public boolean isExpression()
  {
    return true;
  }

  public static QualifiedName make( String str )
  {
    String pstr = "package " + str + ";";
    QualifiedName ret;

    try{
      CompilationUnit cu = PartialParser.makeCompilationUnit( pstr );

      ImportStatementList islist = cu.getImportStatements();
      TypeDeclarationList tdlist = cu.getTypeDeclarations();
      if(! (islist.isEmpty() && tdlist.isEmpty())){
	throw new PtreeException();
      }

      ret = cu.getPackage();
    }catch( Exception ex ){
      return null;
    }

    return ret;
  }

  public void writeCode()
  {
    writeDebugL();

    super.writeCode();

    writeDebugR();
  }

  /**
   * Gets the specified element at
   * This causes side-effect.
   *
   * @param  n  
   */
  public Identifier elementAt( int n )
  {
    return (Identifier) ptreelist.elementAt( n );
  }

  /**
   * Sets the specified element at the specified place of the list
   *
   * @param  p  element
   * @param  n  the number where to set element
   */
  public void setElementAt( Identifier p, int n )
  {
    ptreelist.setElementAt( p, n );
  }

  /**
   * Adds the specified element after the list
   *
   * @param  p  Identifier to be inserted into the list
   */
  public void addElement( Identifier p )
  {
    ptreelist.addElement( p );
  }

  /**
   * Inserts the specified element into the list
   * before the specified element of the list.
   *
   * @param  p  the element to be inserted into the list
   * @param  n  number of the element before which insertion ocuurs
   */
  public void insertElementAt( Identifier p, int n )
  {
    ptreelist.insertElementAt( p, n );
  }

  /**
   * Returns the first element
   *
   * @returns  the first element
   */
  public Identifier getFirst()
  {
    return (Identifier) ptreelist.getFirst();
  }

  /**
   * Returns the new list from this list's element except for
   * the first element
   * This does't cause side-effect as a list.
   *
   * @returns the rest list except for the first element
   */
  public QualifiedName getRest()
  {
    QualifiedName ret = new QualifiedName();
    for( int i = 1; i < getLength(); i++ ){
      ret.addElement( elementAt( i ) );
    }

    return ret;
  }

  public Identifier lastElement()
  {
    return (Identifier) ptreelist.lastElement();
  }

  public void setLastElement( Identifier ident )
  {
    setElementAt( ident, getLength() - 1 );
  }

  /**
   * Appends a list after this list.
   * This doesn't cause side-effect as a list.
   *
   * @param  lst  a list to be appended
   */
  public void append( QualifiedName lst )
  {
    for( int i = 0; i < lst.getLength(); i++ ){
      ptreelist.addElement( lst.elementAt( i ) );
    }
  }

}
