/*
 * SpecialCallStatement.java 1.0
 *
 * This supports JDK1.1 language specification
 * like "hoge.super();".
 *
 * Oct 12, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 12, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The SpecialCallStatement is not a regular statement.
 * The place where this is put on is restricted to the first line
 * of constructor body.
 */
public class SpecialCallStatement extends NonLeaf
{
  public SpecialCallStatement(
      QualifiedName   qname,
      SpecialName     sp,
      ExpressionList  exprlist )
  {
    super();
    if(qname == null){
      qname = new QualifiedName();
    }
    if(exprlist == null){
      exprlist = new ExpressionList();
    }
    set( qname, sp, exprlist );
  }

  SpecialCallStatement()
  {
    super();
  }
  
  public void writeCode()
  {
    writeTab();  writeDebugL();

    QualifiedName qname = getClassName();
    if(qname.getLength() != 0){
      qname.writeCode();
      out.print( "." );
    }

    SpecialName sp = getSpecialName();
    sp.writeCode();
    
    out.print( "(" );

    ExpressionList el = getArguments();
    if(el.isEmpty()){
      el.writeCode();
    }else{
      out.print( " " );
      el.writeCode();
      out.print( " " );
    }

    out.print( ");" );

    writeDebugR();  out.println();
  }
  
  public QualifiedName getClassName()
  {
    return (QualifiedName) elementAt( 0 );
  }

  public SpecialName getSpecialName()
  {
    return (SpecialName) elementAt( 1 );
  }
  
  public ExpressionList getArguments()
  {
    return (ExpressionList) elementAt( 2 );
  }

}
