/*
 * StaticInitilizer.java 1.0
 *
 *
 * Jun 20, 1997
 * Sep 29, 1997
 * Oct 10, 1997
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The StaticInitilizer class presents static initializer node
 * of parse tree, which is like:
 * <br><blockquote><pre>
 *     static{
 *         field1 = ...
 *     }
 * </pre></blockquote><br>
 *
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.MemberDeclaration
 * @see openjava.ptree.StatementList
 */
public class StaticInitializer extends NonLeaf
    implements MemberDeclaration
{
  /**
   * Constructs new StaticInitializer form its elements.
   *
   * @param  stmts  statement list
   */
  public StaticInitializer( StatementList stmts )
  {
    super();
    if(stmts == null){
      stmts = new StatementList();
    }
    set( stmts );
  }

  StaticInitializer()
  {
    super();
  }
  
  public void writeCode()
  {
    openjava.util.Comment comment = getComment();
    if(comment != null){
      comment.writeCode();
    }

    writeTab();  writeDebugL();

    out.println( "static{" );

    StatementList block = getBody();
    if(block != null){    
      pushNest();  block.writeCode();  popNest();
    }

    writeTab();  out.print( "}" );  writeDebugR();
  }
  
  /**
   * Gets the body of static Initializer block.
   *
   * @return  statement list.
   */
  public StatementList getBody()
  {
    return (StatementList) elementAt( 0 );
  }

  /**
   * Sets the body of static Initializer block.
   *
   * @param  stmts  statement list to set.
   */
  public void setBody( StatementList stmts )
  {
    setElementAt( stmts, 0 );
  }

}
