/*
 * TypeDeclaration.java 1.0
 *
 * Jun 11, 1997  by mich
 * Sep 22, 1997  by Bv.
 * 
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated: Sep 27, 1997
 * @author  Michiaki Tatsubori
 ***/
package openjava.ptree;


/**
 * The TypeDeclaration interface presents a common interfaces
 * to both InterfaceDeclaration class and ClassDeclaration class.
 * 
 * This interface is implemented by
 *   InterfaceDeclaration
 *   ClassDeclaration
 * 
 * @see openjava.ptree.ClassDeclaration
 * @see openjava.ptree.InterfaceDeclaration
 * @see openjava.ptree.FieldDeclaration
 */
public interface TypeDeclaration
    extends Statement, MemberDeclaration, Ptree
{
  /**
   * Sets the modifier list which modifies this declaration.
   *
   * @param  mlst  the modifier list
   */
  public void setModifiers( ModifierList mlst );

  /**
   * Obtains the modifier list which modifies this declaration.
   *
   * @return  the modifier list
   */
  public ModifierList getModifiers();

  /**
   * Sets the identifier of this class/interface's name.
   * 
   * @param  ident the identifier indicating this class/interface name
   */
  public void setName( Identifier ident );

  /**
   * Obtains the identifier of this class/interface's name.
   * 
   * @return  the identifier indicating this class/interface name.
   */
  public Identifier getName();

  /**
   * Obtains the member list of this class or interface.
   *
   * @return  the member list.
   */
  public MemberDeclarationList getBody();

  /**
   * Sets the member list of this class or interface.
   *
   * @param  mdecls  the member list to set.
   */
  public void setBody( MemberDeclarationList mdecls );

  public void setMetaclass( openjava.Class metaclazz );
  public openjava.Class getMetaclass();

}
