/*
 * TypeSpecifier.java 1.0
 *
 * This interface is made to type ptree-node into the type
 * specifier in the body of class.
 *
 * Jun 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 11, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 11, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The <code>TypeSpecifier</code> class represents a type specifier
 * node of parse tree.
 *
 * @see openjava.ptree.Ptree
 * @see openjava.ptree.NonLeaf
 */
public class TypeSpecifier extends NonLeaf
{
  private UserDecoratorList userdecorator = null;

  int dim;
  
  /**
   * Allocates a new object.
   *
   * @param typename type name
   * @param n array dimension
   */
  public TypeSpecifier( TypeName typename, int n )
  {
    super();
    set( (Ptree) typename );
    setDim( n );
  }

  /**
   * Allocates a new object.
   * No array dimension will set.
   * <br><blockquote><pre>
   *     new TypeSpecifier( typename )
   * </pre></blockquote><br>
   * equals:
   * <br><blockquote><pre>
   *     new TypeSpecifier( typename, 0 )
   * </pre></blockquote><br>
   *
   * @param typename type name
   */
  public TypeSpecifier( TypeName typename )
  {
    this( typename, 0 );
  }

  TypeSpecifier() {
    super();
  }

  public Ptree makeRecursiveCopy() {
    TypeSpecifier result
        = (TypeSpecifier) super.makeRecursiveCopy();
    result.dim = this.dim;
    return result;
  }

  public Ptree makeCopy() {
    TypeSpecifier result
        = (TypeSpecifier) super.makeCopy();
    result.dim = this.dim;
    return result;
  }
  
  public void writeCode()
  {
    writeDebugL();

    TypeName typename = getTypeName();
    typename.writeCode();
    
    int n = getDim();
    for( int i = 0; i < n; i++ ){
      out.print( "[]" );
    }
    
    writeDebugR();
  }
  
  /**
   * Gets array dimension of declarated type
   *
   * @return array dimension
   */
  public int getDim()
  {
    return dim;
  }
  
  /**
   * Sets array dimension of declarated type
   *
   * @param dim array dimension
   */
  public void setDim( int n )
  {
    dim = n;
  }
  
  /**
   * Gets the type name of this type specifier.
   *
   * @return  the type name.
   */
  public TypeName getTypeName()
  {
    return (TypeName) elementAt( 0 );
  }

  /**
   * Sets the type name of this type specifier.
   *
   * @param typename the type name to set.
   */
  public void setTypeName( TypeName typename )
  {
    setElementAt( typename, 0 );
  }

  /**
   * Sets the user decorator list.
   *
   * @return  the user decorator list.
   */
  public void setUserDecorators( UserDecoratorList ud )
  {
    if(ud == null)  ud = new UserDecoratorList();
    this.userdecorator = ud;
  }

  /**
   * Gets the user decorator list.
   *
   * @param typename the type name to set.
   */
  public UserDecoratorList getUserDecorators()
  {
    return this.userdecorator;
  }

}
