/*
 * VariableDeclarator.java 1.0
 *
 * This interface is made to type ptree-node into the variable
 * declarator in the body of class.
 *
 * Jun 20, 1997 by mich
 * Aug 20, 1997 by mich
 * Sep 29, 1997 by bv
 * Oct 10, 1997 by mich
 *
 * @see openjava.ptree.Ptree
 * @version 1.0 last updated:  Oct 10, 1997
 * @author  Michiaki Tatsubori
 */
package openjava.ptree;


import java.lang.String;
import java.io.OutputStream;
import openjava.main.SymbolTable;


/**
 * The VariableDeclarator class presents variable declarator node
 * of parse tree
 * 
 * @see openjava.ptree.NonLeaf
 * @see openjava.ptree.Identifier
 * @see openjava.ptree.VariableInitializer
 */
public class VariableDeclarator extends NonLeaf
{
  /**
   * Allocates a new object.
   *
   * @param  declname  the declarator name.
   * @param  varinit  the variable initializer.
   */
  public VariableDeclarator(
      Identifier declname,
      VariableInitializer varinit )
  {
    super();
    set( declname, (Ptree) varinit );
  }

  VariableDeclarator() {
    super();
  }
  
  /**
   * write code
   *
   * Identifier "=" VaraiableInitializer
   */
  public void writeCode()
  {
    Identifier declname = getVariable();
    declname.writeCode();
    
    VariableInitializer varinit = getInitializer();
    if(varinit != null){
      out.print( " = " );
      varinit.writeCode();
    }
  }
  
  /**
   * Gets declarator name, declarator name includes variable name
   * but its dimension.
   * 
   * @return declarator name
   */
  public Identifier getVariable()
  {
    return (Identifier) elementAt( 0 );
  }

  /**
   * Sets declarator name, declarator name includes variable name
   * but its dimension.
   * 
   * @param  name  declarator name to set.
   * @see  openjava.ptree.TypeSpecifier
   */
  public void setVariable( Identifier name )
  {
    setElementAt( name, 0 );
  }
  
  /**
   * Gets variable initializer.
   *
   * @return variable initializer.
   */
  public VariableInitializer getInitializer()
  {
    return (VariableInitializer) elementAt( 1 );
  }

  /**
   * Sets variable initializer.
   *
   * @param  vinit  the variable initializer to set.
   */
  public void setInitializer( VariableInitializer vinit )
  {
    setElementAt( vinit, 1 );
  }

}
