/*
 * WrappedClass.java 1.0
 *
 * May 14, 1997 by mich
 * Sep 28, 1997 by mich
 * Oct 11, 1997 by mich
 *
 * @version 1.0  last update:  Oct 11, 1997
 * @author Michiaki Tatsubori
 */
package openjava.tools;


import java.lang.*;
import java.lang.reflect.*;
import openjava.util.PartialParser;


/**
 * The WrappedClass class will be the base class of openjava.Class.
 * This class has the same methods as java.lang.Class.
 *
 * @see java.lang.Class
 * @see openjava.Class
 */
public class WrappedClass extends PartialParser
{
  
  /**
   * This class cannot be construct.
   */
  protected WrappedClass()
  {
  }

  /**
   * Returns the <code>Class</code> object associated with the class 
   * with the given string name. 
   * Given the fully-qualified name for a class or interface, this
   * method attempts to locate, load and link the class.  If it
   * succeeds, returns the Class object representing the class.  If
   * it fails, the method throws a ClassNotFoundException.
   * <br>
   * For example, the following code fragment returns the runtime 
   * <code>Class</code> descriptor for the class named 
   * <code>java.lang.Thread</code>: 
   * <ul><code>
   *   Class&nbsp;t&nbsp;= Class.forName("java.lang.Thread")
   * </code></ul>
   *
   * @param      className   the fully qualified name of the desired class.
   * @return     the <code>Class</code> descriptor for the class with the
   *             specified name.
   * @exception  ClassNotFoundException  if the class could not be found.
   * @since      JDK1.1
   */
  public static Class forName( String className )
    throws ClassNotFoundException
  {
    return Class.forName( className );
  }

}
