/*
 * openjava.lex 1.0
 *
 * The following will be resolved by the <code>Scanner</code> class.
 * "[]" can be "[  ]"
 * ".class" can be ".  class "
 *
 * @author Michiaki Tatsubori
 * @version 1.0  May 10, 1997
 */
package openjava.main;
import java.lang.String;
import java.lang.System;
import java.util.Vector;
import java.util.Enumeration;


public class Yylex {
	private final int YY_BUFFER_SIZE = 512;
	private final int YY_F = -1;
	private final int YY_NO_STATE = -1;
	private final int YY_NOT_ACCEPT = 0;
	private final int YY_START = 1;
	private final int YY_END = 2;
	private final int YY_NO_ANCHOR = 4;
	private final byte YYEOF = -1;

  protected int           commentNest = 0;
  protected Vector        javadocComments = null;
  protected StringBuffer  commentLine = null;
  protected Yytoken mkRet( int tokenID ) {
    Yytoken ret = new Yytoken( tokenID, yytext(),
			       yyline, yychar,
			       javadocComments );
    javadocComments = null;
    return ret;
  }
  private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character.",
    "Error: Unexpected error."
  };
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 
  public static final int E_UNEXPECTED = 4; 
  public void error( int code )
  {
    System.err.println( errorMsg[code] );
    System.err.println( "text=" + yytext() +
                        " line=" + yyline + " char=" + yychar );    
    throw( new Error( "Lexical Error: assertion failed." ) );
  }
	private java.io.DataInputStream yy_instream;
	private int yy_buffer_index;
	private int yy_buffer_read;
	private int yy_buffer_start;
	private int yy_buffer_end;
	private byte yy_buffer[];
	private int yychar;
	private int yyline;
	private int yy_lexical_state;
	public Yylex (java.io.InputStream instream) {
		if (null == instream) {
			throw (new Error("Error: Bad input stream initializer."));
		}
		yy_instream = new java.io.DataInputStream(instream);
		yy_buffer = new byte[YY_BUFFER_SIZE];
		yy_buffer_read = 0;
		yy_buffer_index = 0;
		yy_buffer_start = 0;
		yy_buffer_end = 0;
		yychar = 0;
		yyline = 0;
		yy_lexical_state = YYINITIAL;

  commentNest = 0;
	}
	private boolean yy_eof_done = false;
	private void yy_do_eof () {
		if (false == yy_eof_done) {

  if(yy_lexical_state != YYINITIAL){
    error( E_ENDCOMMENT );
  }
		}
		yy_eof_done = true;
	}
	private final int COMMENT = 1;
	private final int YYINITIAL = 0;
	private final int LINE = 3;
	private final int DOCU = 2;
	private final int yy_state_dtrans[] = {
		0,
		212,
		214,
		216
	};
	private void yybegin (int state) {
		yy_lexical_state = state;
	}
	private byte yy_advance ()
		throws java.io.IOException {
		int next_read;
		int i;
		int j;

		if (yy_buffer_index < yy_buffer_read) {
			return yy_buffer[yy_buffer_index++];
		}

		if (0 != yy_buffer_start) {
			i = yy_buffer_start;
			j = 0;
			while (i < yy_buffer_read) {
				yy_buffer[j] = yy_buffer[i];
				++i;
				++j;
			}
			yy_buffer_end = yy_buffer_end - yy_buffer_start;
			yy_buffer_start = 0;
			yy_buffer_read = j;
			yy_buffer_index = j;
			next_read = yy_instream.read(yy_buffer,
					yy_buffer_read,
					yy_buffer.length - yy_buffer_read);
			if (-1 == next_read) {
				return YYEOF;
			}
			yy_buffer_read = yy_buffer_read + next_read;
		}

		while (yy_buffer_index >= yy_buffer_read) {
			if (yy_buffer_index >= yy_buffer.length) {
				yy_buffer = yy_double(yy_buffer);
			}
			next_read = yy_instream.read(yy_buffer,
					yy_buffer_read,
					yy_buffer.length - yy_buffer_read);
			if (-1 == next_read) {
				return YYEOF;
			}
			yy_buffer_read = yy_buffer_read + next_read;
		}
		return yy_buffer[yy_buffer_index++];
	}
	private void yy_move_start () {
		if ((byte) '\n' == yy_buffer[yy_buffer_start]
			|| (byte) '\r' == yy_buffer[yy_buffer_start]) {
			++yyline;
		}
		++yychar;
		++yy_buffer_start;
	}
	private void yy_pushback () {
		--yy_buffer_end;
	}
	private void yy_mark_start () {
		int i;
		for (i = yy_buffer_start; i < yy_buffer_index; ++i) {
			if ((byte) '\n' == yy_buffer[i] || (byte) '\r' == yy_buffer[i]) {
				++yyline;
			}
		}
		yychar = yychar
			+ yy_buffer_index - yy_buffer_start;
		yy_buffer_start = yy_buffer_index;
	}
	private void yy_mark_end () {
		yy_buffer_end = yy_buffer_index;
	}
	private void yy_to_mark () {
		yy_buffer_index = yy_buffer_end;
	}
	private java.lang.String yytext () {
		return (new java.lang.String(yy_buffer,
			yy_buffer_start,
			yy_buffer_end - yy_buffer_start));
	}
	private int yylength () {
		return yy_buffer_end - yy_buffer_start;
	}
	private byte[] yy_double (byte buf[]) {
		int i;
		byte newbuf[];
		newbuf = new byte[2*buf.length];
		for (i = 0; i < buf.length; ++i) {
			newbuf[i] = buf[i];
		}
		return newbuf;
	}
	private final int YY_E_INTERNAL = 0;
	private final int YY_E_MATCH = 1;
	private java.lang.String yy_error_string[] = {
		"Error: Internal error.\n",
		"Error: Unmatched input.\n"
	};
	private void yy_error (int code,boolean fatal) {
		java.lang.System.out.print(yy_error_string[code]);
		java.lang.System.out.flush();
		if (fatal) {
			throw new Error("Fatal Error.\n");
		}
	}
	private int yy_acpt[] = {
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NOT_ACCEPT,
		YY_NOT_ACCEPT,
		YY_NOT_ACCEPT,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR,
		YY_NO_ANCHOR
	};
	private int yy_cmap[] = {
		0, 0, 0, 0, 0, 0, 0, 0,
		1, 1, 2, 0, 1, 3, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		1, 4, 5, 0, 6, 7, 8, 9,
		10, 11, 12, 13, 14, 15, 16, 17,
		18, 19, 19, 19, 20, 20, 20, 20,
		21, 21, 22, 23, 24, 25, 26, 27,
		0, 28, 28, 28, 29, 30, 29, 6,
		6, 6, 6, 6, 31, 6, 6, 6,
		6, 6, 6, 6, 6, 6, 6, 6,
		32, 6, 6, 33, 34, 35, 36, 6,
		0, 37, 38, 39, 40, 41, 42, 43,
		44, 45, 46, 47, 48, 49, 50, 51,
		52, 6, 53, 54, 55, 56, 57, 58,
		59, 60, 61, 62, 63, 64, 65, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0,
		0 
	};
	private int yy_rmap[] = {
		0, 1, 2, 3, 4, 5, 6, 7,
		1, 1, 8, 9, 1, 10, 11, 12,
		13, 1, 1, 14, 15, 16, 1, 17,
		1, 18, 1, 19, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 1, 1,
		1, 20, 21, 22, 1, 23, 24, 1,
		25, 1, 1, 1, 26, 1, 1, 27,
		5, 1, 1, 1, 1, 28, 29, 1,
		1, 30, 5, 31, 5, 5, 1, 5,
		5, 5, 5, 5, 5, 5, 5, 5,
		5, 5, 5, 5, 5, 5, 32, 5,
		5, 5, 33, 5, 1, 5, 5, 5,
		5, 5, 5, 5, 5, 5, 5, 5,
		5, 5, 5, 5, 5, 5, 5, 5,
		5, 5, 5, 5, 1, 5, 1, 1,
		1, 1, 34, 1, 1, 35, 36, 36,
		1, 37, 38, 1, 1, 1, 1, 1,
		39, 40, 1, 1, 41, 42, 3, 43,
		1, 44, 45, 46, 47, 48, 49, 50,
		51, 52, 53, 54, 55, 56, 57, 58,
		59, 60, 61, 62, 63, 64, 65, 66,
		67, 68, 69, 70, 71, 72, 73, 74,
		75, 76, 77, 78, 79, 80, 81, 82,
		83, 84, 67, 85, 86, 87, 88, 89,
		90, 91, 92, 93, 94, 95, 96, 97,
		98, 99, 100, 101, 102, 103, 104, 105,
		106, 107, 108, 109, 110, 111, 112, 113,
		114, 115, 116, 117, 118, 119, 120, 121,
		122, 123, 124, 125, 126, 127, 128, 129,
		130, 131, 132, 133, 134, 135, 136, 137,
		138, 139, 140, 141, 142, 143, 144, 145,
		146, 147, 148, 149, 150, 151, 152, 153,
		154, 155, 156, 157, 158, 159, 160, 161,
		162, 163, 164, 165, 166, 167, 168, 169,
		170, 171, 172, 173, 174, 175, 176, 177,
		178, 179, 180, 181, 182, 183, 184, 185,
		186, 187, 188, 189, 190, 191, 192, 193,
		194, 195, 196, 197, 198, 199, 200, 201,
		202, 203, 204, 205, 206, 207, 208, 209,
		210, 211, 212, 213, 214, 215, 216, 217,
		218, 219, 220, 221, 222, 223, 224, 225,
		226, 227, 228, 229, 230, 231, 232, 233,
		234, 235, 236, 237, 238, 239, 240, 241,
		242, 243, 244, 245, 246, 247, 248, 249,
		250, 251, 252, 253, 254, 255, 256, 257,
		258, 259, 260, 261, 262, 263, 264, 265,
		266, 267, 268, 269, 270, 271, 272 
	};
	private int yy_nxt[][] = {
		{ 1, 2, 3, 128, 4, 127, 5, 6,
			7, 141, 8, 9, 10, 11, 12, 13,
			14, 15, 16, 130, 130, 130, 17, 18,
			19, 20, 21, 22, 5, 5, 5, 5,
			5, 23, 1, 24, 25, 366, 280, 284,
			129, 285, 232, 286, 5, 143, 5, 5,
			287, 372, 235, 5, 343, 344, 320, 237,
			5, 288, 321, 5, 5, 5, 26, 27,
			28, 29 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, 2, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, 128, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 30, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 32, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			33, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 34, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 35, -1, -1, -1, -1, -1, -1,
			-1, 36, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 37, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 38, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 39,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 40, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 41, 41, 41, 41, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 154,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 42, -1, -1, -1,
			-1, 43, -1, -1, -1, -1, -1, -1,
			-1, 44, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			45, -1, 46, 46, 46, 130, -1, -1,
			-1, -1, -1, -1, -1, 47, 156, 144,
			158, -1, -1, -1, -1, -1, -1, -1,
			47, 156, 47, -1, -1, -1, -1, -1,
			144, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 158, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			48, 49, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 50, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 51, 52, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 53, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 54, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 57, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 58,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 41, 41, 41, 41, -1, -1,
			-1, -1, -1, -1, -1, 131, 168, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			131, 168, 131, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 60, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 172, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 145, 145, 145, 145, -1, -1,
			-1, -1, -1, -1, -1, 132, 174, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			132, 174, 132, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			45, -1, 46, 46, 46, 130, -1, -1,
			-1, -1, -1, -1, -1, 47, 156, 133,
			-1, -1, -1, -1, -1, -1, -1, -1,
			47, 156, 47, -1, -1, -1, -1, -1,
			133, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 63, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 64, 65, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			298, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 61, 61, 61, 61, -1, -1,
			-1, -1, -1, -1, -1, 134, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			134, -1, 134, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 62, 62, 62, 62, -1, -1,
			-1, -1, -1, -1, 62, 62, 62, 135,
			-1, -1, -1, -1, -1, 62, 62, 62,
			62, 62, 62, -1, -1, -1, -1, -1,
			135, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 70, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 352, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			219, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 100, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, 138, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, 139, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ 126, 126, 126, 126, 126, 31, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126, 140, 126, 126, 126, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126, 126, 126, 126, 126, 126, 126,
			126, 126 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 346, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 55, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			45, -1, 130, 130, 130, 130, -1, -1,
			-1, -1, -1, -1, -1, 47, 156, 144,
			-1, -1, -1, -1, -1, -1, -1, -1,
			47, 156, 47, -1, -1, -1, -1, -1,
			144, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 119, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 123, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, 126, -1, -1,
			-1, 126, -1, -1, -1, -1, -1, -1,
			-1, -1, 126, 126, 126, 160, -1, -1,
			-1, -1, -1, -1, 160, 160, 160, -1,
			-1, -1, 126, -1, -1, 160, 126, 160,
			160, 160, 126, -1, -1, -1, -1, -1,
			-1, -1, 126, -1, -1, 126, -1, 126,
			162, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ 149, 149, 149, 149, 149, 149, 149, 149,
			149, -1, 149, 149, 149, 149, 149, 149,
			149, 149, 149, 149, 149, 149, 149, 149,
			149, 149, 149, 149, 149, 149, 149, 149,
			149, 149, 152, 149, 149, 149, 149, 149,
			149, 149, 149, 149, 149, 149, 149, 149,
			149, 149, 149, 149, 149, 149, 149, 149,
			149, 149, 149, 149, 149, 149, 149, 149,
			149, 149 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 56, 5, 5, 5, 5, 5,
			5, 322, 153, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 145, 145, 145, 145, -1, -1,
			-1, -1, -1, -1, -1, 131, 186, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			131, 186, 131, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 146, 146, 146, 146, -1, -1,
			-1, -1, -1, -1, -1, 132, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			132, -1, 132, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 120, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 124, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 66, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 151, 151, 151, 151, -1, -1,
			-1, -1, -1, -1, -1, 131, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			131, -1, 131, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, 149, -1, -1,
			-1, 149, -1, -1, -1, -1, -1, -1,
			-1, -1, 164, 164, 283, 281, -1, -1,
			-1, -1, -1, -1, 281, 281, 281, -1,
			-1, -1, 149, -1, -1, 281, 166, 281,
			281, 281, 166, -1, -1, -1, -1, -1,
			-1, -1, 149, -1, -1, 149, -1, 149,
			282, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 367, 67,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			170, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 68, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 176, -1, 176,
			-1, -1, 61, 61, 61, 61, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 361, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			175, 5, 5, 5, 69, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 62, 62, 62, 62, -1, -1,
			-1, -1, -1, -1, 62, 62, 62, -1,
			-1, -1, -1, -1, -1, 62, 62, 62,
			62, 62, 62, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 71, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 233, 233, 233, 233, -1, -1,
			-1, -1, -1, -1, 233, 233, 233, -1,
			-1, -1, -1, -1, -1, 233, 233, 233,
			233, 233, 233, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 72, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 160, 160, 160, 160, -1, -1,
			-1, -1, -1, -1, 160, 160, 160, -1,
			-1, -1, -1, -1, -1, 160, 160, 160,
			160, 160, 160, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			162, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 73, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 178, 178, 178, 236, -1, -1,
			-1, -1, -1, -1, 236, 236, 236, -1,
			-1, -1, -1, -1, -1, 236, 236, 236,
			236, 236, 236, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 74, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 236, 236, 236, 236, -1, -1,
			-1, -1, -1, -1, 236, 236, 236, -1,
			-1, -1, -1, -1, -1, 236, 236, 236,
			236, 236, 236, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 75, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 180, -1, 180,
			-1, -1, 151, 151, 151, 151, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 76, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 182, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			77, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 184, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 78, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 188, -1, 188,
			-1, -1, 146, 146, 146, 146, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 79, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 61, 61, 61, 61, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			80, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 192, 192, 192, 194, -1, -1,
			-1, -1, -1, -1, 194, 194, 194, -1,
			-1, -1, -1, -1, -1, 194, 194, 194,
			194, 194, 194, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 81,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 151, 151, 151, 151, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 82, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 196, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 83, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 198,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 84,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 85, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 146, 146, 146, 146, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			86, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 126, 126, 126, 126, -1, -1,
			-1, -1, -1, -1, 126, 126, 126, -1,
			-1, -1, -1, -1, -1, 126, 126, 126,
			126, 126, 126, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 87,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 149, 149, 149, 149, -1, -1,
			-1, -1, -1, -1, 149, 149, 149, -1,
			-1, -1, -1, -1, -1, 149, 149, 149,
			149, 149, 149, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 88,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 149, 149, 149, 149, -1, -1,
			-1, -1, -1, -1, 149, 149, 149, -1,
			-1, -1, -1, -1, -1, 149, 149, 149,
			149, 149, 149, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 89, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 92, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 90, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 200, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 91, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 202, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 93, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 204, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 94,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 206, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 95, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, 208, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 96,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 210,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 97, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 116,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 98,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ 118, 118, 142, 128, 118, 118, 118, 118,
			118, 118, 118, 118, 136, 118, 118, 118,
			118, 147, 118, 118, 118, 118, 118, 118,
			118, 118, 118, 118, 118, 118, 118, 118,
			118, 118, 118, 118, 118, 118, 118, 118,
			118, 118, 118, 118, 118, 118, 118, 118,
			118, 118, 118, 118, 118, 118, 118, 118,
			118, 118, 118, 118, 118, 118, 118, 118,
			118, 118 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 99, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ 121, 121, 122, 138, 121, 121, 121, 121,
			121, 121, 121, 121, 137, 121, 121, 121,
			121, 148, 121, 121, 121, 121, 121, 121,
			121, 121, 121, 121, 121, 121, 121, 121,
			121, 121, 121, 121, 121, 121, 121, 121,
			121, 121, 121, 121, 121, 121, 121, 121,
			121, 121, 121, 121, 121, 121, 121, 121,
			121, 121, 121, 121, 121, 121, 121, 121,
			121, 121 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 101, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ 216, 216, 125, 139, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216, 216, 216, 216, 216, 216, 216,
			216, 216 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 102,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 103, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 104, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 105, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 106, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 107,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 108, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 109, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 110, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			111, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 112,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 113, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 114, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 115,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			117, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 292, 5, 5,
			5, 5, 5, 5, 5, 293, 5, 5,
			294, 5, 5, 150, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 190, 190, 190, 190, -1, -1,
			-1, -1, -1, -1, 190, 190, 190, -1,
			-1, -1, -1, -1, -1, 190, 190, 190,
			190, 190, 190, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 194, 194, 194, 194, -1, -1,
			-1, -1, -1, -1, 194, 194, 194, -1,
			-1, -1, -1, -1, -1, 194, 194, 194,
			194, 194, 194, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 323, 5, 5,
			5, 155, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			244, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 194, 194, 194, 194, -1, -1,
			-1, -1, -1, -1, 194, 194, 194, -1,
			-1, -1, -1, -1, -1, 194, 194, 194,
			194, 194, 194, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 245, 5, 5, 5,
			5, 5, 5, 5, 5, 157, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 159,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 161, 248,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 163, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 165, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 167,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 169, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			171, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 173, 5, 5,
			5, 5, 5, 5, 5, 256, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 177, 5, 5,
			351, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 179, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 181,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 183, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 185, 334,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 187, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 189, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 191, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 193, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 195, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 197, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			199, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			201, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 203, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 205, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 207, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 209, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 211, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 213,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 215, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			217, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			218, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 220, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 221,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 222,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			223, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			224, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 225,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 226, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 227, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 228,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 229, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 230,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 231, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 345, 5, 289, 5, 5,
			5, 5, 5, 5, 238, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 236, 236, 236, 236, -1, -1,
			-1, -1, -1, -1, 236, 236, 236, -1,
			-1, -1, -1, -1, -1, 236, 236, 236,
			236, 236, 236, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 281, 281, 281, 281, -1, -1,
			-1, -1, -1, -1, 281, 281, 281, -1,
			-1, -1, -1, -1, -1, 281, 281, 281,
			281, 281, 281, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			282, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 59, -1, -1, -1, -1, -1, -1,
			-1, -1, 234, 234, 234, 236, -1, -1,
			-1, -1, -1, -1, 236, 236, 236, -1,
			-1, -1, -1, -1, -1, 236, 236, 236,
			236, 236, 236, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 239, 5, 5,
			5, 5, 5, 5, 240, 5, 5, 5,
			290, 5, 5, 291, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			241, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 347, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 242, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 243, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 246, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 247, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 249, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 250, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			251, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 252, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 253, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 254, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 255, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 257, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 258, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			362, 5, 5, 259, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 260, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			261, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			262, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 263,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 264,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 265, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			266, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 267, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 268, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 269, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 270, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 271, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 272, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 273, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 274,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 275, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 276, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 277, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 278, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 279, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 295, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 326,
			296, 5, 327, 5, 374, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 297, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 299, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 300,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 301, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 302,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 303, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 304, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			305, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 306, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 307, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 308,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 309, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 310, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 311, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 312,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 313, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 314,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 315, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 316, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 317,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 318, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 319, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 348, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 349, 5, 5,
			324, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 325, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 328, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 329, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 330,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 331,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 332, 5, 5,
			5, 5, 5, 360, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 333,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 335, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 336, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 337, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 338, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 339, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 340, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 341, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 342, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 350, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 353,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 354, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 355, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 356, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 357, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 358, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 359, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 363,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 364, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 365, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 368,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 369, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 370, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 371,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		},
		{ -1, -1, -1, -1, -1, -1, 5, -1,
			-1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, 5, 5, 5, -1, -1,
			-1, -1, -1, -1, 5, 5, 5, 5,
			5, -1, -1, -1, -1, 5, 5, 5,
			5, 5, 5, 5, 5, 5, 5, 5,
			5, 5, 373, 5, 5, 5, 5, 5,
			5, 5, 5, 5, 5, 5, -1, -1,
			-1, -1 
		}
	};
	public Yytoken yylex ()
		throws java.io.IOException {
		byte yy_lookahead;
		int yy_anchor = YY_NO_ANCHOR;
		int yy_state = yy_state_dtrans[yy_lexical_state];
		int yy_next_state = YY_NO_STATE;
		int yy_last_accept_state = YY_NO_STATE;
		boolean yy_initial = true;
		int yy_this_accept;

		yy_mark_start();
		yy_this_accept = yy_acpt[yy_state];
		if (YY_NOT_ACCEPT != yy_this_accept) {
			yy_last_accept_state = yy_state;
			yy_mark_end();
		}
		while (true) {
			yy_lookahead = yy_advance();
			yy_next_state = YY_F;
			if (YYEOF != yy_lookahead) {
				yy_next_state = yy_nxt[yy_rmap[yy_state]][yy_cmap[((int)yy_lookahead)&255]];
			}
			if (YY_F != yy_next_state) {
				yy_state = yy_next_state;
				yy_initial = false;
				yy_this_accept = yy_acpt[yy_state];
				if (YY_NOT_ACCEPT != yy_this_accept) {
					yy_last_accept_state = yy_state;
					yy_mark_end();
				}
			}
			else {
				if (YYEOF == yy_lookahead && true == yy_initial) {
					yy_do_eof();
					return null;
				}
				else if (YY_NO_STATE == yy_last_accept_state) {
					throw (new Error("Lexical Error: Unmatched Input."));
				}
				else {
					yy_to_mark();
					yy_anchor = yy_acpt[yy_last_accept_state];
					if (0 != (YY_END & yy_anchor)) {
						yy_pushback();
					}
					if (0 != (YY_START & yy_anchor)) {
						yy_move_start();
					}
					switch (yy_last_accept_state) {
					case 1:
						{ error( E_UNMATCHED ); }
					case -2:
						break;
					case 2:
						{ }
					case -3:
						break;
					case 3:
						{ }
					case -4:
						break;
					case 4:
						{ return mkRet( SymbolTable.NOT ); }
					case -5:
						break;
					case 5:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -6:
						break;
					case 6:
						{ return mkRet( SymbolTable.MOD ); }
					case -7:
						break;
					case 7:
						{ return mkRet( SymbolTable.BITAND ); }
					case -8:
						break;
					case 8:
						{ return mkRet( SymbolTable.LPAREN ); }
					case -9:
						break;
					case 9:
						{ return mkRet( SymbolTable.RPAREN ); }
					case -10:
						break;
					case 10:
						{ return mkRet( SymbolTable.TIMES ); }
					case -11:
						break;
					case 11:
						{ return mkRet( SymbolTable.PLUS ); }
					case -12:
						break;
					case 12:
						{ return mkRet( SymbolTable.COMMA ); }
					case -13:
						break;
					case 13:
						{ return mkRet( SymbolTable.MINUS ); }
					case -14:
						break;
					case 14:
						{ return mkRet( SymbolTable.DOT ); }
					case -15:
						break;
					case 15:
						{ return mkRet( SymbolTable.DIVIDE ); }
					case -16:
						break;
					case 16:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -17:
						break;
					case 17:
						{ return mkRet( SymbolTable.COLON ); }
					case -18:
						break;
					case 18:
						{ return mkRet( SymbolTable.SEMI ); }
					case -19:
						break;
					case 19:
						{ return mkRet( SymbolTable.LESS ); }
					case -20:
						break;
					case 20:
						{ return mkRet( SymbolTable.EQUALS ); }
					case -21:
						break;
					case 21:
						{ return mkRet( SymbolTable.GREATER ); }
					case -22:
						break;
					case 22:
						{ return mkRet( SymbolTable.QUESTION ); }
					case -23:
						break;
					case 23:
						{ return mkRet( SymbolTable.LBRACK ); }
					case -24:
						break;
					case 24:
						{ return mkRet( SymbolTable.RBRACK ); }
					case -25:
						break;
					case 25:
						{ return mkRet( SymbolTable.XOR ); }
					case -26:
						break;
					case 26:
						{ return mkRet( SymbolTable.LCURLY ); }
					case -27:
						break;
					case 27:
						{ return mkRet( SymbolTable.BITOR ); }
					case -28:
						break;
					case 28:
						{ return mkRet( SymbolTable.RCURLY ); }
					case -29:
						break;
					case 29:
						{ return mkRet( SymbolTable.BITNOT ); }
					case -30:
						break;
					case 30:
						{ return mkRet( SymbolTable.OP_NE ); }
					case -31:
						break;
					case 31:
						{ return mkRet( SymbolTable.STRING ); }
					case -32:
						break;
					case 32:
						{ return mkRet( SymbolTable.ASS_MOD ); }
					case -33:
						break;
					case 33:
						{ return mkRet( SymbolTable.OP_LAND ); }
					case -34:
						break;
					case 34:
						{ return mkRet( SymbolTable.ASS_AND ); }
					case -35:
						break;
					case 35:
						{ error( E_ENDCOMMENT ); }
					case -36:
						break;
					case 36:
						{ return mkRet( SymbolTable.ASS_MUL ); }
					case -37:
						break;
					case 37:
						{ return mkRet( SymbolTable.OP_INC ); }
					case -38:
						break;
					case 38:
						{ return mkRet( SymbolTable.ASS_ADD ); }
					case -39:
						break;
					case 39:
						{ return mkRet( SymbolTable.OP_DEC ); }
					case -40:
						break;
					case 40:
						{ return mkRet( SymbolTable.ASS_SUB ); }
					case -41:
						break;
					case 41:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -42:
						break;
					case 42:
						{
	  yybegin(COMMENT);
	  commentNest++;
	}
					case -43:
						break;
					case 43:
						{ yybegin(LINE); }
					case -44:
						break;
					case 44:
						{ return mkRet( SymbolTable.ASS_DIV ); }
					case -45:
						break;
					case 45:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -46:
						break;
					case 46:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -47:
						break;
					case 47:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -48:
						break;
					case 48:
						{ return mkRet( SymbolTable.OP_SHL ); }
					case -49:
						break;
					case 49:
						{ return mkRet( SymbolTable.OP_LE ); }
					case -50:
						break;
					case 50:
						{ return mkRet( SymbolTable.OP_EQ ); }
					case -51:
						break;
					case 51:
						{ return mkRet( SymbolTable.OP_GE ); }
					case -52:
						break;
					case 52:
						{ return mkRet( SymbolTable.OP_SHR ); }
					case -53:
						break;
					case 53:
						{ return mkRet( SymbolTable.OP_EMPTYDIM ); }
					case -54:
						break;
					case 54:
						{ return mkRet( SymbolTable.ASS_XOR ); }
					case -55:
						break;
					case 55:
						{ return mkRet( SymbolTable.DO ); }
					case -56:
						break;
					case 56:
						{ return mkRet( SymbolTable.IF ); }
					case -57:
						break;
					case 57:
						{ return mkRet( SymbolTable.ASS_OR ); }
					case -58:
						break;
					case 58:
						{ return mkRet( SymbolTable.OP_LOR ); }
					case -59:
						break;
					case 59:
						{ return mkRet( SymbolTable.CHARACTER ); }
					case -60:
						break;
					case 60:
						{
	  yybegin(DOCU);
	  commentNest++;
	  javadocComments = new Vector();
	  commentLine = new StringBuffer( yytext() );
	}
					case -61:
						break;
					case 61:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -62:
						break;
					case 62:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -63:
						break;
					case 63:
						{ return mkRet( SymbolTable.ASS_SHL ); }
					case -64:
						break;
					case 64:
						{ return mkRet( SymbolTable.ASS_SHR ); }
					case -65:
						break;
					case 65:
						{ return mkRet( SymbolTable.OP_SHRR ); }
					case -66:
						break;
					case 66:
						{ return mkRet( SymbolTable.FOR ); }
					case -67:
						break;
					case 67:
						{ return mkRet( SymbolTable.INT ); }
					case -68:
						break;
					case 68:
						{ return mkRet( SymbolTable.NEW ); }
					case -69:
						break;
					case 69:
						{ return mkRet( SymbolTable.TRY ); }
					case -70:
						break;
					case 70:
						{ return mkRet( SymbolTable.ASS_SHRR ); }
					case -71:
						break;
					case 71:
						{ return mkRet( SymbolTable.BYTE ); }
					case -72:
						break;
					case 72:
						{ return mkRet( SymbolTable.CASE ); }
					case -73:
						break;
					case 73:
						{ return mkRet( SymbolTable.CHAR ); }
					case -74:
						break;
					case 74:
						{ return mkRet( SymbolTable.ELSE ); }
					case -75:
						break;
					case 75:
						{ return mkRet( SymbolTable.GOTO ); }
					case -76:
						break;
					case 76:
						{ return mkRet( SymbolTable.LONG ); }
					case -77:
						break;
					case 77:
						{ return mkRet( SymbolTable.NULL ); }
					case -78:
						break;
					case 78:
						{ return mkRet( SymbolTable.THIS ); }
					case -79:
						break;
					case 79:
						{ return mkRet( SymbolTable.TRUE ); }
					case -80:
						break;
					case 80:
						{ return mkRet( SymbolTable.VOID ); }
					case -81:
						break;
					case 81:
						{ return mkRet( SymbolTable.BREAK ); }
					case -82:
						break;
					case 82:
						{ return mkRet( SymbolTable.CATCH ); }
					case -83:
						break;
					case 83:
						{ return mkRet( SymbolTable.CLASS ); }
					case -84:
						break;
					case 84:
						{ return mkRet( SymbolTable.CONST ); }
					case -85:
						break;
					case 85:
						{ return mkRet( SymbolTable.FALSE ); }
					case -86:
						break;
					case 86:
						{ return mkRet( SymbolTable.FINAL ); }
					case -87:
						break;
					case 87:
						{ return mkRet( SymbolTable.FLOAT ); }
					case -88:
						break;
					case 88:
						{ return mkRet( SymbolTable.SHORT ); }
					case -89:
						break;
					case 89:
						{ return mkRet( SymbolTable.SUPER ); }
					case -90:
						break;
					case 90:
						{ return mkRet( SymbolTable.THROW ); }
					case -91:
						break;
					case 91:
						{ return mkRet( SymbolTable.WHILE ); }
					case -92:
						break;
					case 92:
						{ return mkRet( SymbolTable.DOTCLASS ); }
					case -93:
						break;
					case 93:
						{ return mkRet( SymbolTable.DOUBLE ); }
					case -94:
						break;
					case 94:
						{ return mkRet( SymbolTable.IMPORT ); }
					case -95:
						break;
					case 95:
						{ return mkRet( SymbolTable.NATIVE ); }
					case -96:
						break;
					case 96:
						{ return mkRet( SymbolTable.PUBLIC ); }
					case -97:
						break;
					case 97:
						{ return mkRet( SymbolTable.RETURN ); }
					case -98:
						break;
					case 98:
						{ return mkRet( SymbolTable.STATIC ); }
					case -99:
						break;
					case 99:
						{ return mkRet( SymbolTable.SWITCH ); }
					case -100:
						break;
					case 100:
						{ return mkRet( SymbolTable.THROWS ); }
					case -101:
						break;
					case 101:
						{ return mkRet( SymbolTable.BOOLEAN ); }
					case -102:
						break;
					case 102:
						{ return mkRet( SymbolTable.DEFAULT ); }
					case -103:
						break;
					case 103:
						{ return mkRet( SymbolTable.EXTENDS ); }
					case -104:
						break;
					case 104:
						{ return mkRet( SymbolTable.FINALLY ); }
					case -105:
						break;
					case 105:
						{ return mkRet( SymbolTable.PACKAGE ); }
					case -106:
						break;
					case 106:
						{ return mkRet( SymbolTable.PRIVATE ); }
					case -107:
						break;
					case 107:
						{ return mkRet( SymbolTable.ABSTRACT ); }
					case -108:
						break;
					case 108:
						{ return mkRet( SymbolTable.CONTINUE ); }
					case -109:
						break;
					case 109:
						{ return mkRet( SymbolTable.VOLATILE ); }
					case -110:
						break;
					case 110:
						{ return mkRet( SymbolTable.INTERFACE ); }
					case -111:
						break;
					case 111:
						{ return mkRet( SymbolTable.PROTECTED ); }
					case -112:
						break;
					case 112:
						{ return mkRet( SymbolTable.TRANSIENT ); }
					case -113:
						break;
					case 113:
						{ return mkRet( SymbolTable.IMPLEMENTS ); }
					case -114:
						break;
					case 114:
						{ return mkRet( SymbolTable.INSTANCEOF ); }
					case -115:
						break;
					case 115:
						{ return mkRet( SymbolTable.METAOBJECT ); }
					case -116:
						break;
					case 116:
						{ return mkRet( SymbolTable.METAOBJECT ); }
					case -117:
						break;
					case 117:
						{ return mkRet( SymbolTable.SYNCHRONIZED ); }
					case -118:
						break;
					case 118:
						{ }
					case -119:
						break;
					case 119:
						{
	  commentNest--;
	  if(commentNest < 0)  error( E_UNEXPECTED );
	  if(commentNest == 0)  yybegin(YYINITIAL);
	}
					case -120:
						break;
					case 120:
						{ commentNest++; }
					case -121:
						break;
					case 121:
						{
	  commentLine.append( yytext() );
	}
					case -122:
						break;
					case 122:
						{
	  javadocComments.addElement( commentLine );
	  commentLine = new StringBuffer();
	}
					case -123:
						break;
					case 123:
						{
	  commentNest--;
	  if(commentNest < 0)  error( E_UNEXPECTED );
	  commentLine.append( yytext() );
	  if(commentNest == 0){
	    javadocComments.addElement( commentLine.toString() );
	    commentLine = null;
	    yybegin(YYINITIAL);
          }
	}
					case -124:
						break;
					case 124:
						{
	  commentNest++;
	  commentLine.append( yytext() );
	}
					case -125:
						break;
					case 125:
						{ yybegin(YYINITIAL); }
					case -126:
						break;
					case 127:
						{ error( E_UNMATCHED ); }
					case -127:
						break;
					case 128:
						{ }
					case -128:
						break;
					case 129:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -129:
						break;
					case 130:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -130:
						break;
					case 131:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -131:
						break;
					case 132:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -132:
						break;
					case 133:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -133:
						break;
					case 134:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -134:
						break;
					case 135:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -135:
						break;
					case 136:
						{ }
					case -136:
						break;
					case 137:
						{
	  commentLine.append( yytext() );
	}
					case -137:
						break;
					case 138:
						{
	  javadocComments.addElement( commentLine );
	  commentLine = new StringBuffer();
	}
					case -138:
						break;
					case 139:
						{ yybegin(YYINITIAL); }
					case -139:
						break;
					case 141:
						{ error( E_UNMATCHED ); }
					case -140:
						break;
					case 142:
						{ }
					case -141:
						break;
					case 143:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -142:
						break;
					case 144:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -143:
						break;
					case 145:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -144:
						break;
					case 146:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -145:
						break;
					case 147:
						{ }
					case -146:
						break;
					case 148:
						{
	  commentLine.append( yytext() );
	}
					case -147:
						break;
					case 150:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -148:
						break;
					case 151:
						{ return mkRet( SymbolTable.NUMBER ); }
					case -149:
						break;
					case 153:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -150:
						break;
					case 155:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -151:
						break;
					case 157:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -152:
						break;
					case 159:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -153:
						break;
					case 161:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -154:
						break;
					case 163:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -155:
						break;
					case 165:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -156:
						break;
					case 167:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -157:
						break;
					case 169:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -158:
						break;
					case 171:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -159:
						break;
					case 173:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -160:
						break;
					case 175:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -161:
						break;
					case 177:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -162:
						break;
					case 179:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -163:
						break;
					case 181:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -164:
						break;
					case 183:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -165:
						break;
					case 185:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -166:
						break;
					case 187:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -167:
						break;
					case 189:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -168:
						break;
					case 191:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -169:
						break;
					case 193:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -170:
						break;
					case 195:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -171:
						break;
					case 197:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -172:
						break;
					case 199:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -173:
						break;
					case 201:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -174:
						break;
					case 203:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -175:
						break;
					case 205:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -176:
						break;
					case 207:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -177:
						break;
					case 209:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -178:
						break;
					case 211:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -179:
						break;
					case 213:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -180:
						break;
					case 215:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -181:
						break;
					case 217:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -182:
						break;
					case 218:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -183:
						break;
					case 219:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -184:
						break;
					case 220:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -185:
						break;
					case 221:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -186:
						break;
					case 222:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -187:
						break;
					case 223:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -188:
						break;
					case 224:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -189:
						break;
					case 225:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -190:
						break;
					case 226:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -191:
						break;
					case 227:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -192:
						break;
					case 228:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -193:
						break;
					case 229:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -194:
						break;
					case 230:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -195:
						break;
					case 231:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -196:
						break;
					case 232:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -197:
						break;
					case 235:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -198:
						break;
					case 237:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -199:
						break;
					case 238:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -200:
						break;
					case 239:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -201:
						break;
					case 240:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -202:
						break;
					case 241:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -203:
						break;
					case 242:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -204:
						break;
					case 243:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -205:
						break;
					case 244:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -206:
						break;
					case 245:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -207:
						break;
					case 246:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -208:
						break;
					case 247:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -209:
						break;
					case 248:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -210:
						break;
					case 249:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -211:
						break;
					case 250:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -212:
						break;
					case 251:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -213:
						break;
					case 252:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -214:
						break;
					case 253:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -215:
						break;
					case 254:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -216:
						break;
					case 255:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -217:
						break;
					case 256:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -218:
						break;
					case 257:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -219:
						break;
					case 258:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -220:
						break;
					case 259:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -221:
						break;
					case 260:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -222:
						break;
					case 261:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -223:
						break;
					case 262:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -224:
						break;
					case 263:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -225:
						break;
					case 264:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -226:
						break;
					case 265:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -227:
						break;
					case 266:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -228:
						break;
					case 267:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -229:
						break;
					case 268:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -230:
						break;
					case 269:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -231:
						break;
					case 270:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -232:
						break;
					case 271:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -233:
						break;
					case 272:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -234:
						break;
					case 273:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -235:
						break;
					case 274:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -236:
						break;
					case 275:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -237:
						break;
					case 276:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -238:
						break;
					case 277:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -239:
						break;
					case 278:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -240:
						break;
					case 279:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -241:
						break;
					case 280:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -242:
						break;
					case 284:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -243:
						break;
					case 285:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -244:
						break;
					case 286:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -245:
						break;
					case 287:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -246:
						break;
					case 288:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -247:
						break;
					case 289:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -248:
						break;
					case 290:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -249:
						break;
					case 291:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -250:
						break;
					case 292:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -251:
						break;
					case 293:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -252:
						break;
					case 294:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -253:
						break;
					case 295:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -254:
						break;
					case 296:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -255:
						break;
					case 297:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -256:
						break;
					case 298:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -257:
						break;
					case 299:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -258:
						break;
					case 300:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -259:
						break;
					case 301:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -260:
						break;
					case 302:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -261:
						break;
					case 303:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -262:
						break;
					case 304:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -263:
						break;
					case 305:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -264:
						break;
					case 306:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -265:
						break;
					case 307:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -266:
						break;
					case 308:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -267:
						break;
					case 309:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -268:
						break;
					case 310:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -269:
						break;
					case 311:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -270:
						break;
					case 312:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -271:
						break;
					case 313:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -272:
						break;
					case 314:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -273:
						break;
					case 315:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -274:
						break;
					case 316:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -275:
						break;
					case 317:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -276:
						break;
					case 318:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -277:
						break;
					case 319:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -278:
						break;
					case 320:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -279:
						break;
					case 321:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -280:
						break;
					case 322:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -281:
						break;
					case 323:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -282:
						break;
					case 324:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -283:
						break;
					case 325:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -284:
						break;
					case 326:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -285:
						break;
					case 327:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -286:
						break;
					case 328:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -287:
						break;
					case 329:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -288:
						break;
					case 330:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -289:
						break;
					case 331:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -290:
						break;
					case 332:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -291:
						break;
					case 333:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -292:
						break;
					case 334:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -293:
						break;
					case 335:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -294:
						break;
					case 336:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -295:
						break;
					case 337:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -296:
						break;
					case 338:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -297:
						break;
					case 339:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -298:
						break;
					case 340:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -299:
						break;
					case 341:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -300:
						break;
					case 342:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -301:
						break;
					case 343:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -302:
						break;
					case 344:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -303:
						break;
					case 345:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -304:
						break;
					case 346:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -305:
						break;
					case 347:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -306:
						break;
					case 348:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -307:
						break;
					case 349:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -308:
						break;
					case 350:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -309:
						break;
					case 351:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -310:
						break;
					case 352:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -311:
						break;
					case 353:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -312:
						break;
					case 354:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -313:
						break;
					case 355:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -314:
						break;
					case 356:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -315:
						break;
					case 357:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -316:
						break;
					case 358:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -317:
						break;
					case 359:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -318:
						break;
					case 360:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -319:
						break;
					case 361:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -320:
						break;
					case 362:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -321:
						break;
					case 363:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -322:
						break;
					case 364:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -323:
						break;
					case 365:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -324:
						break;
					case 366:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -325:
						break;
					case 367:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -326:
						break;
					case 368:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -327:
						break;
					case 369:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -328:
						break;
					case 370:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -329:
						break;
					case 371:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -330:
						break;
					case 372:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -331:
						break;
					case 373:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -332:
						break;
					case 374:
						{ return mkRet( SymbolTable.IDENTIFIER ); }
					case -333:
						break;
					default:
						yy_error(YY_E_INTERNAL,false);
					case -1:
					}
					yy_initial = true;
					yy_state = yy_state_dtrans[yy_lexical_state];
					yy_next_state = YY_NO_STATE;
					yy_last_accept_state = YY_NO_STATE;
					yy_mark_start();
					yy_this_accept = yy_acpt[yy_state];
					if (YY_NOT_ACCEPT != yy_this_accept) {
						yy_last_accept_state = yy_state;
					}
				}
			}
		}
	}
}
